/*
 *
 * File: AppArguments.java
 * Date: Mar 9, 2009; 2:49:21 PM
 */
package de.desy.mst.libs.accelerator;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Logger;

import de.desy.mst.libs.accelerator.number.DBoolean;
import de.desy.mst.libs.accelerator.number.DDouble;
import de.desy.mst.libs.accelerator.number.DFloat;
import de.desy.mst.libs.accelerator.number.DInteger;
import de.desy.mst.libs.accelerator.number.DLong;
import de.desy.mst.libs.accelerator.number.INumber;

/**
 * Application command line parameters read from the command line interface. 
 * 
 * Establish the default value for the following keywords:
 * 
 * {@link CGeneralConstant.CLA_ARMED1}, false
 * {@link CGeneralConstant.CLA_TAB}, 0
 * {@link CGeneralConstant.CLA_INITDELAY}, 15
 * {@link CGeneralConstant.CLA_MODE}, "BETRIEB"
 * {@link CGeneralConstant.CLA_LOG}, "INFO"
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: AppArguments.java 63081 2014-11-13 17:50:45Z walla $
 */
public final class AppArguments
{
	// --- CLASS FIELDS --------------------------------------------------------
	private final static Logger log = LogProvider.getLogger(AppArguments.class);
	private final static Map<String, INumber> cmdNumbers = new HashMap<String, INumber>();
	private final static Map<String, String> cmdString = new HashMap<String, String>();
	
	static
	{
		setNumber(CGeneralConstant.CLA_ARMED1, new DBoolean(false));
		setNumber(CGeneralConstant.CLA_TAB, new DInteger(0));
		setNumber(CGeneralConstant.CLA_INITDELAY, new DInteger(15));

		setString(CGeneralConstant.CLA_MODE, "BETRIEB");
		setString(CGeneralConstant.CLA_LOG, "INFO");
	}
	
	// --- CLASS METHODS -------------------------------------------------------

	/**
	 * Loads all arguments as one string value into the internal list.
	 * 
	 * @param arg String value with all command line arguments.
	 */
	public static void load(final String args)
	{
		load(args.split("\\s*"));
	}
	
	/**
	 * <p>Loads all arguments into the internal list. Depending on the argument
	 * list the set values reflects the following conversions (remember that the
	 * list of arguments is BEFORE split by a space character from ONE string 
	 * instance):</p>
	 * 
	 * <ul>
	 * 
	 * <li><p>Check if the argument contains the equal sign ('='). If yes split
	 * 		the string value and do the following checking on the second 
	 * 		argument:</p>
	 * 		<ul>
	 *      <li>check if an empty string value is set &rArr; return a <b>boolean</b> 
	 *      	value of {@code TRUE}.</li>
	 *      <li>check if the string equals {@code TRUE} or {@code ON} &rArr;
	 *      	return a <b>boolean</b> value of {@code TRUE}.</li>
	 *      <li>check if the string equals {@code FALSE} or {@code OFF} &rArr;
	 *      	return a <b>boolean</b> value of {@code FALSE}.</li>
	 *      <li>check if the string equals an integer value (from -2^31 to 2^31-1) &rArr;
	 *      	return the corresponding <b>integer</b> value.</li>
	 *      <li>check if the string equals a long value (from -2^63 to 2^63-1) &rArr;
	 *      	return the corresponding <b>long</b> value.</li>
	 *      <li>check if the string equals a double value &rArr;
	 *      	return the corresponding <b>double</b> value.</li>
	 *      <li>otherwise if everything fails return the <b>string</b> value.</li>
	 *      </ul>
	 * </li>
	 * <li>If the argument doesn't contain an equal sign ('=') a <b>boolean</b> 
	 * value of {@code TRUE} is set to the key value.</li> 
	 * </ul>
	 * 
	 * @param args List of all string arguments from the command line.
	 */
	public static void load(final String... args)
	{
		for(final String argument: args)
		{
			final String arg;
			
			if (argument.contains("="))
			{
				final String[] values = argument.trim().split("=");
				
				int idx = 0;
				if (values.length >= 1 && values[0].matches("((-?)|(--?))[a-zA-Z][a-zA-Z0-9_]*"))
				{
					while(values[0].charAt(idx) == '-') idx += 1;
					arg = values[0].substring(idx);
				}
				else
				{
					log.warning("argument cannot be split on the '=' character or doesn't match the pattern: " + argument);
					continue;
				}
				
				final INumber number = getLocalNumber(values[1]);
				
				if (number != null)
					setInternalNumber(arg, number);
				else
					setInternalString(arg, values[1]);
			}
			else
			{
				int idx = 0;
				if (argument.matches("((-?)|(--?))[a-zA-Z][a-zA-Z0-9]*"))
				{
					while(argument.charAt(idx) == '-') idx += 1;
					arg = argument.substring(idx);
				}
				else
				{
					log.warning("argument of " + argument.trim() + " does not match the pattern");
					continue;
				}
				
				setInternalNumber(arg, new DBoolean(true));
			}
		}
	}

	/**
	 * Request if the string value is a number. Also a boolean value is regarded
	 * as a number, where the values of {@code ON} equals to {@code TRUE}
	 * and {@code OFF} equals to {@code FALSE}. If non of the request types
	 * is true (i.e. if it is an integer, long, float, or double value) then 
	 * this method returns with a {@code null}.
	 * 
	 * @param value The requested string value.
	 * @return Returns the transformed number value.
	 */
	private static INumber getLocalNumber(final String value)
	{
		if (value.toUpperCase().equals("TRUE") 
				|| value.toUpperCase().equals("ON"))
			return new DBoolean(true);
		else if (value.toUpperCase().equals("FALSE")
				|| value.toUpperCase().equals("OFF"))
			return new DBoolean(false);
		
		// Checking the value if it has a type of BYTE, SHORT or FLOAT 
		// is not requested here. In all of these cases the values can also be 
		// requested with the widened value of LONG or DOUBLE. The only 
		// exception is the INTEGER value (this comes from the programming 
		// language Java itself).
	
		// check if it is an INTEGER value -------------------------------------
		try
		{
			return new DInteger(Integer.parseInt(value));
		}
		catch(NumberFormatException exception)
		{
		}

		// check if it is a LONG value -----------------------------------------
		try
		{
			return new DLong(Long.parseLong(value));
		}
		catch(NumberFormatException exception)
		{
		}
		
		// check if it is a FLOAT value ---------------------------------------
		try
		{
			return new DFloat(Float.parseFloat(value));
		}
		catch(NumberFormatException exception)
		{
		}
		
		// check if it is a DOUBLE value ---------------------------------------
		try
		{
			return new DDouble(Double.parseDouble(value));
		}
		catch(NumberFormatException exception)
		{
		}
		
		return null;
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/**
	 * Prints all arguments (numeric and string values to the printer stream. If
	 * the printer stream is {@code null} it will print to the console output 
	 * stream.
	 * 
	 * @param stream The printer output stream. 
	 */
	public static void list(final PrintStream stream)
	{
		final PrintStream out; 
		if (stream == null)
		{
			log.warning("stream is null -> set console output stream.");
			out = System.out;
		}
		else
			out = stream;
		
		out.println("----------------------------------------------------");
		out.println("Arguments with a number value:");
		for(String key: cmdNumbers.keySet())
			out.printf("%30s=%20s\n", key, cmdNumbers.get(key).toString());
		out.println("\nArguments with a string value:");
		for(String key: cmdString.keySet())
			out.printf("%30s=%20s\n", key, cmdString.get(key).toString());
		out.println("----------------------------------------------------");
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/**
	 * Returns if one of the internal list contains the argument as a key value.
	 * The key argument is requested in upper case.
	 * 
	 * @return If the internal list contains the argument a {@code true} 
	 * 		otherwise a {@code false} is returned.
	 */
	public static boolean contains(final String arg)
	{
		return cmdNumbers.containsKey(arg.toUpperCase()) | cmdString.containsKey(arg.toUpperCase());
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/**
	 * Request the command line argument name which returns, if the argument
	 * exists the corresponding value. If not a warning 
	 * is printed and a {@code false} is supplied (data type of {@link DBoolean}). 
	 * The request key argument is queried in upper case.
	 * 
	 * @param arg The key argument.
	 * @return The corresponding number value is returned.
	 */ 
	public static INumber getNumber(final String arg)
	{
		if (!checkStringValue(arg))
		{
			log.warning("failed on getting a number value: argument is invalid -> return a boolean value of false");
			return new DBoolean(false);
		}
		else if (!cmdNumbers.containsKey(arg))
		{
			log.warning("argument of '" + arg.trim() + "' doesn't exist - return false.");
			return new DBoolean(false);
		}
		
		return getInternalNumber(arg);
	}
	
	/**
	 * Request the command line argument and looks if the declared class request
	 * the correct type of {@link INumber}. If it exists and the type is reflected
	 * by the argument, then the responding value is returned otherwise a warning 
	 * is logged and a boolean value of {@code false} is supplied (data type of
	 * {@link DBoolean}). The request key argument is queried in upper case.
	 * 
	 * @param arg The key argument.
	 * @param clazz Class data type of the request argument.
	 * @return The corresponding number value is returned.
	 */ 
	public static INumber getNumber(final String arg, final Class<? extends INumber> clazz)
	{
		if (!checkStringValue(arg))
		{
			log.warning("failed on getting a number value: argument is invalid -> return a boolean value of false");
			return new DBoolean(false);
		}
		else if (!cmdNumbers.containsKey(arg))
		{
			log.warning("argument of '" + arg.trim() + "' doesn't exist - return a boolean value of false");
			return new DBoolean(false);
		}
		else if (clazz != null && !clazz.equals(getInternalNumber(arg).getClass()))
			log.warning("request number of " + clazz.getSimpleName() + " doesn't reflect the expected argument type " + getInternalNumber(arg).getClass().getSimpleName());

		return getInternalNumber(arg);
	}

	private static INumber getInternalNumber(final String arg)
	{
		return cmdNumbers.get(arg.toUpperCase());
	}

	/**
	 * Sets the current number with the argument of a key. The key argument is 
	 * stored in upper case.
	 * 
	 * @param arg The key string argument.
	 * @param value The corresponding value parameter.
	 */
	public static void setNumber(final String arg, final INumber value)
	{
		if (!checkStringValue(arg) || !checkNumberValue(value))
		{
			log.warning("failed on setting a number value:" + " argument or value is invalid");
			return;
		}
		
		if (cmdNumbers.containsKey(arg.trim()))
			log.warning("argument of '" + arg.trim() + "' already exist.");
		
		setInternalNumber(arg, value);
	}
	
	private static void setInternalNumber(final String arg, final INumber value)
	{
		cmdNumbers.put(arg.toUpperCase(), value);
	}

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/**
	 * Request the command line interface with the argument name. If it exists
	 * in the internal list a {@link String} is returned otherwise a warning 
	 * is printed and a {@code null} is supplied. The request key value is
	 * queried in upper case.
	 * 
	 * @param arg The String key argument.
	 * @return The corresponding string value is returned.
	 */ 
	public static String getString(final String arg)
	{
		if (!checkStringValue(arg))
		{
			log.warning("failed on getting a string value: argument is invalid -> return an empty string");
			return "";
		}
		else if (!cmdString.containsKey(arg.toUpperCase()))
		{
			log.warning("argument of '" + arg.trim() + "' doesn't exist - return empty string.");
			return "";
		}
		
		return getInternalString(arg);
	}
	
	private static String getInternalString(final String arg)
	{
		return cmdString.get(arg.toUpperCase());
	}

	/**
	 * Sets the current string with the argument of a key. The key argument 
	 * is stored in upper case.
	 * 
	 * @param arg The key string argument.
	 * @param value The corresponding value parameter.
	 */
	public static void setString(final String arg, final String value)
	{
		if (!checkStringValue(arg) || !checkStringValue(value))
		{
			log.warning("failed on setting a string value: argument or value is invalid");
			return;
		}
		else if (cmdString.containsKey(arg.trim().toUpperCase()))
			log.warning("argument of '" + arg.trim() + "' already exist.");

		setInternalString(arg, value);
	}
	
	private static void setInternalString(final String arg, final String value)
	{
		cmdString.put(arg.toUpperCase(), value);
	}

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/**
	 * Checks the string value and throws the expected exception if it is null
	 * or empty.
	 * 
	 * @param value The inspecting string value.
	 * @return Boolean value depend on checking the success.
	 */
	private static boolean checkStringValue(final String value)
	{
		if (value == null || value.isEmpty())
			return false;
		return true;
	}
	
	/**
	 * Checks the number value and throws the expected exception if it is null.
	 * 
	 * @param value The inspecting number value.
	 */
	private static boolean checkNumberValue(final INumber value)
	{
		if (value == null)
			return false;
		return true;
	}
}
/* end of file */