/**
 * File: CounterInstance.java
 * Date: Feb 21, 2013
 */
package de.desy.mst.libs.accelerator;

/**
 * Instance counter. Delivers the same functionality as the global counter
 * {@link CounterGlobal} except that here you have create an instance.
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: CounterInstance.java 53955 2013-02-25 15:47:32Z walla $
 */
public final class CounterInstance {
	// --- CLASS FIELDS --------------------------------------------------------
	public static final int DEFAULT_LIMIT = 3;
	private int limit = CounterInstance.DEFAULT_LIMIT;
	private int counter = 1;

	// --- CONSTRUCTOR ---------------------------------------------------------
	public CounterInstance() { }
	public CounterInstance(final int limit) { 
		if (limit < 1) throw new IllegalArgumentException("limit is lower then 1");
		this.limit = limit;
	}
	
	/**
	 * Resets the counter to 1 and the limit to 3. 
	 * @see #reset(int)
	 */
	public final void reset() {	
		this.counter = 1;
	}
	
	/**
	 * Increments the counter. The counter is not incremented if the counter
	 * exceeds its limit.
	 * @return Returns if the counter exceeds limit a {@code true} otherwise {@code false}.
	 */
	public final boolean increment() {
		// TODO possibly the algorithm can be reduced to:
		//
		//		return (this.limit < this.counter++)
		//
		// or
		// 
		//		try { return this.limit < this.counter; } finally { counter++; }
		//
		if (this.limit < this.counter)
			return true;
		this.counter += 1;
		return false;
	}

	/**
	 * Returns {@code true} if the counter exceeds its limit otherwise
	 * {@code false}. The counter is not manipulated by this method.
	 * @return Depending on the relationship between the limit and counter the
	 * 		corresponding boolean value is returned. 
	 */
	public final boolean isFinished() { return (this.limit <= this.counter); }
	
	/**
	 * Delivers if this counter has any effects on the {@link #increment()} method
	 * or not or, in other words: if this instance is a real counter or a dummy.   
	 * @return Returns the effectiveness of {@link #increment()}
	 */
	public final boolean isCountable() { return this.limit > 1; }
}
/* end of file */