/*
 *
 * File: EName.java
 * Date: Oct 14, 2007; 3:07:54 PM
 */
package de.desy.mst.libs.accelerator;

/**
 * This is the enumeration of different elements which will be returned by the
 * interface method of 
 * {@link de.desy.mst.libs.accelerator.IEEnum#getName(de.desy.mst.libs.accelerator.EName)}.
 * It will return according to this enumeration the corresponding string value.
 * If the enumeration indeed exits but is not used a 
 * {@link de.desy.mst.libs.accelerator.exception.EnumerationNotUsedException}.
 *
 * <table cellspacing="2" cellpadding="4">
 * <tr><th>Enumeration</th><th>Description</th></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link EName#RANGE}</td><td>The range between the bounding CDI data elements. This is truly the listed elements in the defined order in the concrete project!</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link EName#PROPERTY}</td><td>On not CDI used data elements, this returns the property name. Mostly this doesn't exist on CDI used data elements.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link EName#DESCRIPTION}</td><td>The description of the data element.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link EName#PRECISION}</td><td>If it exists, it is the precision of the element.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link EName#PRE_TINEADDRESS}</td><td>The TINE pre address string. This will always include the context and server name and sometimes the device name (mostly on TINE paths which are not covered by CDI).</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link EName#BUTTON}</td><td>The button text to be displayed in the GUI element.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link EName#DEVICENAME}</td><td>On CDI data elements, this returns the device name. Mostly this doesn't exist on not CDI used data elements.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link EName#ID}</td><td>Is a unique identifier per enumeration entry.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link EName#NAME_TABLE}</td><td>The table or element name, which is displayed in the GUI. Sometimes this can also be with a HTML tagged string.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link EName#NAME}</td><td>The title name, which is displayed in the GUI. Sometimes this can also be a HTML tagged string.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link EName#NAME_CAVITY}</td><td>The cavity name, which is displayed in the GUI. Sometimes this can also be with a HTML tagged string.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link EName#UNIT}</td><td>If it exists it is the unit of the data element.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link EName#VALUE}</td><td>The value. <b>THIS WILL BE DELETED IN FUTURE VERSIONS. DON'T USE THIS!</b></td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link EName#CHANNEL}</td><td>Normally this is only used for Linac2 for reading the CDI multiplexer values of RF. Other application will not use this entry.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link EName#STATUS_ON}</td><td>Value if the status is ON (depends on the returned TINE value, but is mostly an integer value of 1).</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link EName#STATUS_OFF}</td><td>Value if the status is OFF (depends on the returned TINE value, but is mostly an integer value of 0).</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link EName#DRAG_AND_DROP}</td><td>Returns a what so ever object for the Drag & Drop facility of the Swing components.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link EName#FULL_TINE_ADDRESS}</td><td>Full TINE address, including the context and server name, possibly the device name and the property name.</td></tr>
 * </table>
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: EName.java 34784 2010-04-01 16:23:29Z walla $
 */
public enum EName
{
	/**
	 * <p>Is the unique id of the current enumeration entry. Normally it will give
	 * back the current enumeration the string value which is unique. It does 
	 * this by calling the internal method of every enumeration type 
	 * {@code toString}.</p>
	 * <p>This is currently <b>deprecated</b> because you can also get
	 * a unified value by returning the string value of the enumeration {@link Object#toString()}
	 * and therefore does the same.</p>
	 */
	@Deprecated
	ID,

	/** A short description of this entry. */
	DESCRIPTION,
	
	/**
	 * Returns the TINE pre-string address element. This is mostly the TINE context, server
	 * and the device name, but can also include the property name */
	PRE_TINEADDRESS,
	
	/**
	 * The full TINE address, including the context, server name, possibly
	 * the device name and property name.
	 */
	FULL_TINE_ADDRESS,
	
	/** TINE device name. */
	DEVICENAME,

	/**
	 * Returns the property of the enumeration of the TINE property called
	 * {@link de.desy.mst.libs.accelerator.model.ETineProperty}.
	 */
	PROPERTY,
	
	/** Precision of this data element. */
	PRECISION,

	/**
	 * <p>Returns the range. These sets two values of 
	 * {@link de.desy.mst.libs.accelerator.IEData} which returns the
	 * number and sets them as the first and last value, i.e. the range of it.
	 * <b>This is truly the listed elements in the defined order in the concrete 
	 * project!</b></p>
	 * 
	 * <p>The range of the TINE device names is given by excluding it from the 
	 * initialized map as the first and last bordering elements connection with
	 * a sign, in this case a hyphen ("-"). If there exists only one TINE device 
	 * name it will simply return the device name as a string value.</p>
	 * <p>Examples:
	 * <ul>
	 * <li><tt>CavVoltage</tt> on <tt>RF125-PIA.CDI</tt>: will only access the corresponding TINE 
	 * element.</li>
	 * <li><tt>UAnode - CavVoltage</tt> on <tt>RF125-PIA.CDI</tt>: will access all elements, 
	 * starting from the element of <tt>UAnode</tt> till the last element 
	 * called <tt>CavVoltage</tt>.</li>
	 * </ul>
	 * </p>
	 */
	RANGE,

	/** Button text field. */
	BUTTON,
	
	/** Name. */
	NAME,
	
	/** Table name. */
	NAME_TABLE,
	
	/** Cavity name. */
	NAME_CAVITY,

	/** Unit of this data element. */
	UNIT,
	
	/** Value of this data element. */
	@Deprecated	VALUE,
	
	/** Status value is ON. */
	STATUS_ON,
	
	/** Status value is OFF. */
	STATUS_OFF,
	
	/** A Drag & Drop facility of a Swing component. */
	DRAG_AND_DROP
}
/* end of file */