/*
 *
 * File: AlarmControl.java
 * Date: Aug 4, 2008; 10:10:29 AM
 */
package de.desy.mst.libs.accelerator.data;

import de.desy.mst.auxiliary.devl.deviceserverlib.alarms.Alarm;
import de.desy.mst.libs.accelerator.IEDeviceGroup;

/**
 * The alarm control class.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: AlarmControl.java 71377 2016-05-23 11:25:57Z walla $
 */
public final class AlarmControl
{
    private final IEDeviceGroup _devices;
	private final float _alarmValue;
	private final int _maxCounter;
	private final Alarm _alarm;
	private final boolean _retrieveAlarm;

	// -------------------------------------------------------------------------
	private long _counter;

	// -------------------------------------------------------------------------

	/**
	 * The minimal {@code AlarmControl} constructor. Only takes two arguments
	 * and sets the rest to their default values.
	 *
	 * @param maxCounter_ Maximum counter. Can also be 0, so it must be fired
	 * 		immediately.
	 * @param alarm_ The alarm which is for example send to the alarm
	 * 		server.
	 */
	public AlarmControl(
			final int maxCounter_,
			final Alarm alarm_)
	{
		_devices = null;
		_alarmValue = 0.0f;
		_counter = 0;
		_maxCounter = maxCounter_;
		_alarm = alarm_;
		_retrieveAlarm = true;

	}

	/**
	 * Constructor.
	 *
	 * @param devices_ Device group.
	 * @param alarmValue_ Alarm value which causes the alarm.
	 * @param alarm_ The alarm which is for example send to the alarm
	 * 		server.
	 */
	public AlarmControl(
			final IEDeviceGroup devices_,
			final float alarmValue_,
			final Alarm alarm_)
	{
		_devices = devices_;
		_alarmValue = alarmValue_;
		_counter = 0;
		_maxCounter = 0;
		_alarm = alarm_;
		_retrieveAlarm = true;
	}
	
	/**
	 * Constructor.
	 *
	 * @param devices_ Device group.
	 * @param alarmValue_ Alarm value which causes the alarm.
	 * @param maxCounter_ Maximum counter.
	 * @param alarm_ The alarm which is for example send to the alarm
	 * 		server.
	 * @param retrieveAlarm_ The alarm can also be disabled by setting this
	 * 		value to {@code false}.
	 */
	public AlarmControl(
			final IEDeviceGroup devices_,
			final float alarmValue_,
			final int maxCounter_,
			final Alarm alarm_,
			final boolean retrieveAlarm_)
	{
		validate(maxCounter_, alarm_);

		_devices = devices_;
		_alarmValue = alarmValue_;
		_counter = 0;
		_maxCounter = maxCounter_;
		_alarm = alarm_;
		_retrieveAlarm = retrieveAlarm_;
	}

	/**
	 * Constructor.
	 *
	 * @param devices_ Device group.
	 * @param alarmValue_ Alarm value which causes the alarm.
	 * @param maxCounter_ Maximum counter.
	 * @param alarm_ The alarm which is for example send to the alarm
	 * 		server.
	 */
	public AlarmControl(
			final IEDeviceGroup devices_,
			final float alarmValue_,
			final int maxCounter_,
			final Alarm alarm_)
	{
		validate(maxCounter_, alarm_);

		_devices = devices_;
		_alarmValue = alarmValue_;
		_counter = 1;
		_maxCounter = maxCounter_;
		_alarm = alarm_;
		_retrieveAlarm = true;
	}

	/**
	 * Validate values.
	 *
	 * @param maxCounter_ Maximum counter.
	 * @param alarm_ The alarm which is for example send to the alarm
	 * 		server.
	 */
	private void validate(
			final int maxCounter_,
			final Alarm alarm_)
	{
		if (alarm_ == null)
			throw new NullPointerException("Device group is null.");
		if (maxCounter_ <= 0)
			throw new IllegalArgumentException("Maximum counter is illegal: "
					+ maxCounter_);
	}

	/**
	 * Returns the device group.
	 *
	 * @return Returns the device group.
	 */
	public final IEDeviceGroup getDeviceGroup()
	{
		return _devices;
	}

	/**
	 * The alarm on which the error should occur.
	 *
	 * @return Returns the alarm value.
	 */
	public final float getAlarmValue()
	{
		return _alarmValue;
	}
	
	/**
	 * Returns the maximum counter.
	 * @return Maximum counter.
	 */
	public final int getMaxCounter() { return this._maxCounter; }

	/**
	 * Increments the counter.
	 */
	public final void incrementCounter()
	{
		_counter += 1;
	}

	/**
	 * Resets the current counter to 0.
	 */
	public final void resetCounter()
	{
		_counter = 1;
	}

	/**
	 * Requests if the internal counter is greater then the maximum counter.
	 * The maximum counter is set in the constructor.
	 *
	 * @return Returns a {@code true} if the counter exceeds the maximum
	 * 		counter.
	 */
	public final boolean isExceeded()
	{
		return _counter >= _maxCounter;
	}

	/**
	 * Requests if this alarm is retrieved (i.e. enable this alarm) or not.
	 *
	 * @return Returns a {@code false} if this alarm is disabled, otherwise a
	 * 		{@code true} is returned.
	 */
	public final boolean isRetrieveAlarm()
	{
		return _retrieveAlarm;
	}

	/**
	 * Alarm.
	 *
	 * @return Returns the alarm.
	 */
	public final Alarm getAlarm()
	{
		return _alarm;
	}

	/**
	 * Returns the delay time of the TINE link. If now devices are defined in'
	 * the constructor then the default value of 1000 ms is returned.
	 *
	 * @return Returns the delay time.
	 */
	public final int getDelay()
	{
		return (_devices == null) ? 1000 : _devices.delay();
	}
	
	@Override
	public String toString()
	{
		return new StringBuilder().append("AlarmControl[device=").append(this._devices).append(", value=").append(this._alarmValue).append(", counter=").append(this._maxCounter).append(", delay=").append(getDelay()).append(']').toString();
	}

}
/* end of file */