/*
 *
 * File: EErrorExt.java
 * Date: Mar 10, 2008; 3:31:33 PM
 */
package de.desy.mst.libs.accelerator.model;

import java.util.HashMap;
import java.util.Map;
import static de.desy.mst.libs.accelerator.CGeneralConstant.ERROR_BASE;

/**
 * This is the TINE error extension which resolves error codes from the decimal
 * number 2048 (binary 0000.1000.0000.0000, hex 0x0800) on. All these error
 * codes and its messages are also listed in the file
 * /resources/errorCodes/errorCodes.xls).
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: EErrorExt.java 25558 2009-05-19 13:24:51Z walla $
 */
public enum EErrorExt
{
	VALUE_IS_NULL            (ERROR_BASE + 0x00, "Delivered value is null"),
	INVALID_COLLECTION       (ERROR_BASE + 0x01, "Collection of data values is invalid"),
	INVALID_DEVICE_NUMBER    (ERROR_BASE + 0x02, "Device number is invalid"),
	TRANSMITTER_IS_NEAR_MODE (ERROR_BASE + 0x03, "Transmitter is in near mode"),
	UNUSED_VALUE             (ERROR_BASE + 0x04, "Unused value"),
	ARRAY_LENGTH_INVALID     (ERROR_BASE + 0x05, "Array length is invalid"),
	MISLEADING_VALUES        (ERROR_BASE + 0x06, "Misleading values"),
	POINT_OF_DISCONTINUITY   (ERROR_BASE + 0x07, "Point of discontinuity"),
	CODE_ERROR               (ERROR_BASE + 0x08, "Code error"),
	INVALID_NUMBER           (ERROR_BASE + 0x09, "Invalid number"),
	UNDEF_VALUE              (ERROR_BASE + 0x0A, "Undefined value"),
	VALUE_NOT_SET			 (ERROR_BASE + 0x0B, "Value not set"),
	OPERATION_BUSY           (ERROR_BASE + 0x0C, "Operation busy"),
	
	DEAD_VALUE               (ERROR_BASE + 0x0D, "Provided data from the server is not alive")
	;

	/** An associated map which connects the error code with the error
	 * enumeration. */
	private static Map<Integer, EErrorExt> _mapError = new HashMap<Integer, EErrorExt>();

	static {
		for(EErrorExt error: EErrorExt.values())
			_mapError.put(error.getErrorCode(), error);
	}

	/**
	 * Returns on the error code the corresponding error enumeration. If there
	 * doesn't exist an enumeration on the delivered error code then a
	 * {@code null} is returned.
	 *
	 * @param key_ Is the error code.
	 * @return Returns the corresponding error enumeration or {@code null} if it
	 * 		doesn't exist.
	 */
	public static EErrorExt getError(final int key_)
	{
		if (_mapError.containsKey(key_))
			return _mapError.get(key_);
		return null;
	}

	/**
	 * Returns if the corresponding error code is inside my extension ones.
	 *
	 * @param errorCode_ Is the error code.
	 * @return Returns a boolean value which represents if the value is inside
	 * 		the associated map.
	 */
	public static boolean isError(final int errorCode_)
	{
		return _mapError.containsKey(errorCode_);
	}

	/** Is the error code. */
	private final int _errorCode;

	/** Is the error message. */
	private final String _errorMessage;

	/**
	 * The constructor of the enumeration instance.
	 *
	 * @param errorCode_ Is the error code.
	 * @param errorMessage_ Is the error message.
	 */
	private EErrorExt(final int errorCode_, final String errorMessage_)
	{
		if (errorMessage_ == null)
			throw new NullPointerException("The error string value is null.");
		if (errorMessage_.isEmpty())
			throw new IllegalArgumentException("The error string value is" +
					" empty.");

		_errorCode = errorCode_;
		_errorMessage = errorMessage_;
	}

	/**
	 * Returns the error code value.
	 *
	 * @return Returns the error code value
	 */
	public int getErrorCode()
	{
		return _errorCode;
	}

	/**
	 * Returns the error string value.
	 *
	 * @return Returns the error message value.
	 */
	public String getError()
	{
		return _errorMessage;
	}
}
/* end of file */