/*
 *
 * File: CavityHolder.java
 * Date: Jan 27, 2008; 5:22:36 PM
 */
package de.desy.mst.libs.accelerator.model.cavity;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.IEViewGeneric;
import de.desy.mst.libs.accelerator.exception.EnumerationException;
import de.desy.mst.libs.accelerator.exception.EnumerationNotFoundException;

/**
 * <p>This holds all relevant data values for a cavity panel. Each can be get by
 * the method of {@code get(EPanel):ICavityData}. On those you can add new
 * elements to it with the method of {@link ICavityData#put(IECavityElement, IEData)}
 * or get an element by {@link ICavityData#get(IECavityElement)}.</p>
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: CavityHolder.java 22266 2009-02-27 08:56:01Z walla $
 */
public final class CavityHolder {
	// --- INSTANCE FIELDS -----------------------------------------------------
	private ICavityData general;
	private ICavityData element;
	private ICavityData cavitySwitches;
	private final String cavityName;
	
	// --- CONSTRUCTOR ---------------------------------------------------------
	/**
	 * Constructs a new instance of the type {@code CavityHolder}.
	 * @param cavityName Cavity name.
	 */
	public CavityHolder(final String cavityName) {
		this.cavityName = cavityName;
		this.general = new CavityGeneralData();
		this.element = new CavityElementData();
		this.cavitySwitches = new CavitySwitchData();
	}

	/**
	 * Constructs a new instance of the type {@code CavityHolder}.
	 */
	@Deprecated public CavityHolder() {
		this.cavityName = "UNKNOWN";
		this.general = new CavityGeneralData();
		this.element = new CavityElementData();
		this.cavitySwitches = new CavitySwitchData();
	}

	// --- INSTANCE METHODS ----------------------------------------------------
	public String getCavityName() { return this.cavityName; }
	@Override public String toString() { return this.cavityName; }
	
	/**
	 * Returns the corresponding cavity enumeration as a {@code ICavityData}.
	 *
	 * @param cavity_ The viewable enumeration interface.
	 * @return Returns an instance of the {@code ICavityData}.
	 * @throws EnumerationException If the enumeration exists but has no
	 * 		element in this class then a {@code EnumerationException} is thrown.
	 */
	public ICavityData get(final IEViewGeneric cavity_) throws EnumerationException {
		if (!(cavity_ instanceof ECavityView))
			throw new IllegalArgumentException("The delivered argument of" +
					" cavity must be of the type ECavityView.");
		switch((ECavityView) cavity_)
		{
		case GENERAL:
			return this.general;
		case ELEMENT:
			return this.element;
		case SWITCH:
			return this.cavitySwitches;
		default:
			throw new EnumerationNotFoundException("The enumeration of " + cavity_.toString() + " is currently not used.");
		}
	}

	/**
	 *
	 *
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: CavityHolder.java 22266 2009-02-27 08:56:01Z walla $
	 */
	private static abstract class ACavityData implements ICavityData {
		// --- INSTANCE FIELDS -------------------------------------------------
		/** Is the map of the cavity elements. */
		protected Map<IECavityElement, IEData> _mapCavityView = new HashMap<IECavityElement, IEData>();

		// --- INSTANCE METHODS ------------------------------------------------
		@Override public void put(final IECavityElement view_, final IEData data_) {
			this._mapCavityView.put(view_, data_);
		}
		@Override public Collection<IEData> getAllElements() { return this._mapCavityView.values(); }
		@Override public IEData get(final IECavityElement view_) {
			if (view_ == null)
				throw new NullPointerException("Cavity view element is null.");
			return _mapCavityView.get(view_);
		}
	}

	/**
	 * Holds general data and view component of the cavity panel.
	 *
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: CavityHolder.java 22266 2009-02-27 08:56:01Z walla $
	 */
	private static final class CavityGeneralData extends ACavityData implements ICavityData {
		// --- INSTANCE METHODS ------------------------------------------------
		@Override public void put(final IECavityElement view_, final IEData data_) {
			if (data_ == null)
				throw new NullPointerException("Enumeration of the general cavity data is null.");
			if (view_ == null)
				throw new NullPointerException("Enumeration of the general cavity view is null.");
			
			// -----------------------------------------------------------------
			// ATTENTION: Commented out the following statement. A client 
			// project on Petra3 RF uses a different enumeration instead of the 
			// general one. This is NOT included in this library package; 
			// it is a SPECIAL enumeration from a special project which just 
			// uses this library!
//			if (!(view_ instanceof ECavityGeneral))
//				throw new IllegalArgumentException("Parameter of  cavity view" +
//						" is not the type of ECavityGeneral: " + view_);
			// -----------------------------------------------------------------

			super.put(view_, data_);
		}

		@Override public void validateElements() {
			final StringBuffer sb = new StringBuffer();
			for(IECavityElement view: ECavityGeneral.values())
				if (!_mapCavityView.containsKey(view) || _mapCavityView.get(view) == null)
					sb.append(view.toString()).append(' ');
			if (sb.length() != 0)
				throw new IllegalArgumentException("The following cavity view enumerations should be set: " + sb.toString());
		}
	}

	/**
	 * Holds standard data elements and view component of the cavity panel.
	 *
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: CavityHolder.java 22266 2009-02-27 08:56:01Z walla $
	 */
	private static final class CavityElementData extends ACavityData implements ICavityData {
		// --- INSTANCE METHODS ------------------------------------------------
		@Override public void put(final IECavityElement view_, final IEData data_) {
			if (view_ == null)
				throw new NullPointerException("Enumeration of the standard" +
						" cavity view is null.");
			if (data_ == null)
				throw new NullPointerException("Enumeration of the standard" +
						" cavity data is null.");
			if (!(view_ instanceof ECavityElement))
				throw new IllegalArgumentException("Parameter of cavity view" +
						" is not the type of ECavityElement: " + view_);
			super.put(view_, data_);
		}

		@Override public void validateElements() {
			// ATTENTION: A call to this method always successes because the
			// elements of ECavityElement can not be distinguished by used or
			// not used elements, as it is realized with the following commands:
			//
//			StringBuffer sb = new StringBuffer();
//			for(IECavityElement view: ECavityElement.values())
//				if (!_mapCavityView.containsKey(view) ||
//						_mapCavityView.get(view) == null)
//				{
//					sb.append(view.toString());
//					sb.append(' ');
//				}
//			if (sb.length() != 0)
//				throw new IllegalArgumentException("The following cavity view" +
//						" enumerations should be set: " + sb.toString());
		}
	}

	/**
	 * Holds output data and view component of the cavity panel.
	 *
	 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
	 * @version $Id: CavityHolder.java 22266 2009-02-27 08:56:01Z walla $
	 */
	private static final class CavitySwitchData extends ACavityData implements ICavityData {
		// --- INSTANCE METHODS ------------------------------------------------
		@Override public void put(final IECavityElement view_, final IEData data_) {
			if (view_ == null)
				throw new NullPointerException("Enumeration of the switch" +
						" cavity view is null.");
			if (!(view_ instanceof ECavitySwitch))
				throw new IllegalArgumentException("Parameters of cavity view" +
						" is not the type of ECavitySwitch: " + view_);
			super.put(view_, data_);
		}

		@Override public void validateElements() {
			final StringBuffer sb = new StringBuffer();
			for(IECavityElement view: ECavitySwitch.values())
				if (view != ECavitySwitch.U_SOLL_INPUT
						&& view != ECavitySwitch.U_SOLL_OUTPUT
						&& !_mapCavityView.containsKey(view))
					sb.append(view.toString()).append(' ');
			if (sb.length() != 0)
				throw new IllegalArgumentException("The following cavity view" +
						" enumerations should be set: " + sb.toString());
		}
	}
}
/* end of file */