/*
 *
 * File: INumber.java
 * Date: Nov 29, 2006; 4:49:36 PM
 */
package de.desy.mst.libs.accelerator.number;

/**
 * <p>This interface represents every possible primitive number. Because the
 * concrete instance will hold a concrete value there exists two possible 
 * conversion for its type:</p>
 * 
 * <ul>
 * <li>A <i>widening conversion</i> occurs if the value of the converted one
 * has a wider type then before.</li>
 * <li>A <i>narrowing conversion</i> occurs if the value of the converted one
 * has a lower type then before. For this cases some information are lost.</li>
 * </ul>
 * 
 * <p>The following tables gives a description on converting numbers. The 
 * letters have the following meanings: 
 * <dl>
 * <dt><b>/</b></dt><dd>not possible</dd> 
 * <dt><b>N</b></dt><dd>the transformation can not be performed</dd> 
 * <dt><b>Y</b></dt><dd>a widening conversion can be performed</dd> 
 * <dt><b>Y*</b></dt><dd>the conversion is automated be done but some of the least 
 * information can be lost</dd> 
 * <dt><b>C</b></dt><dd>narrowing conversion</dd>
 * </dl><p>
 *
 * <table border="0" type="box" cellpadding="4" cellspacing="4">
 * 
 * <tr bgcolor="FEFEFE"><th>Converted from</th><th colspan="7">Converted to</th></tr>
 * <tr bgcolor="E0E0E0"><th></th><th>boolean</th><th>byte</th><th>short</th><th>integer</th><th>long</th><th>float</th><th>double</th></tr>
 * <tr bgcolor="A0A0A0"><td>boolean</td><td>/</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td></tr>
 * <tr bgcolor="E0E0E0"><td>byte</td>   <td>C</td><td>/</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td></tr>
 * <tr bgcolor="A0A0A0"><td>short</td>  <td>C</td><td>C</td><td>/</td><td>Y</td><td>Y</td><td>Y</td><td>Y</td></tr>
 * <tr bgcolor="E0E0E0"><td>integer</td><td>C</td><td>C</td><td>C</td><td>/</td><td>Y</td><td>Y*</td><td>Y</td></tr>
 * <tr bgcolor="A0A0A0"><td>long</td>   <td>C</td><td>C</td><td>C</td><td>C</td><td>/</td><td>Y*</td><td>Y*</td></tr>
 * <tr bgcolor="E0E0E0"><td>float</td>  <td>C</td><td>C</td><td>C</td><td>C</td><td>C</td><td>/</td><td>Y</td></tr>
 * <tr bgcolor="A0A0A0"><td>double</td> <td>C</td><td>C</td><td>C</td><td>C</td><td>C</td><td>C</td><td>/</td></tr>
 * 
 * </table>
 * 
 * @author <a href="marcus.walla at desy.de">walla</a>
 * @version $Id: INumber.java 22562 2009-03-09 14:31:08Z walla $
 * @see DBoolean
 * @see DByte
 * @see DShort
 * @see DInteger
 * @see DLong
 * @see DFloat
 * @see DDouble
 */
public interface INumber
{
	/**
	 * Returns the boolean value of the corresponding number. This will possibly
	 * transformed to the correct primitive type, but can also concluded not
	 * all possible information - see above.
	 * 
	 * @return Returns the boolean value.
	 */
	public abstract boolean booleanValue();

	/**
	 * Returns the byte value of the corresponding number. This will possibly
	 * transformed to the correct primitive type, but can also concluded not
	 * all possible information - see above.
	 * 
	 * @return Returns the byte value.
	 */
	public abstract byte byteValue();

	/**
	 * Returns the short value of the corresponding number. This will possibly
	 * transformed to the correct primitive type, but can also concluded not
	 * all possible information - see above.
	 * 
	 * @return Returns the short value.
	 */
	public abstract short shortValue();
	
	/**
	 * Returns the integer value of the corresponding number. This will possibly
	 * transformed to the correct primitive type, but can also concluded not
	 * all possible information - see above.
	 * 
	 * @return Returns the integer value.
	 */
	public abstract int intValue();
	
	/**
	 * Returns the long value of the corresponding number. This will possibly
	 * transformed to the correct primitive type, but can also concluded not
	 * all possible information - see above.
	 * 
	 * @return Returns the long value.
	 */
	public abstract long longValue();
	
	/**
	 * Returns the float value of the corresponding number. This will possibly
	 * transformed to the correct primitive type, but can also concluded not
	 * all possible information - see above.
	 * 
	 * @return Returns the float value.
	 */
	public abstract float floatValue();
	
	/**
	 * Returns the double value of the corresponding number. This will possibly
	 * transformed to the correct primitive type, but can also concluded not
	 * all possible information - see above.
	 * 
	 * @return Returns the double value.
	 */
	public abstract double doubleValue();
	
	/**
	 * This will set the current value to the assigned boolean one.
	 * 
	 * @param bool_ The boolean value.
	 */
	public abstract void setValue(boolean bool_);
	
	/**
	 * This will set the current value to the assigned byte one.
	 * 
	 * @param byte_ The byte value.
	 */
	public abstract void setValue(byte byte_);
	
	/**
	 * This will set the current value to the assigned short one.
	 * 
	 * @param short_ The short value.
	 */
	public abstract void setValue(short short_);
	
	/**
	 * This will set the current value to the assigned integer one.
	 * 
	 * @param int_ The integer value.
	 */
	public abstract void setValue(int int_);
	
	/**
	 * This will set the current value to the assigned long one.
	 * 
	 * @param long_ The long value.
	 */
	public abstract void setValue(long long_);
	
	/**
	 * This will set the current value to the assigned float one.
	 * 
	 * @param float_ The float value.
	 */
	public abstract void setValue(float float_);
	
	/**
	 * This will set the current value to the assigned double one.
	 * 
	 * @param double_ The double value.
	 */
	public abstract void setValue(double double_);

}
/* end of file */