/*
 *
 * File: RFSwitchArray.java
 * Date: Jan 25, 2007; 10:07:07 AM
 */
package de.desy.mst.libs.accelerator.view.container;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import de.desy.mst.libs.accelerator.EName;
import de.desy.mst.libs.accelerator.EViewElement;
import de.desy.mst.libs.accelerator.IComponentSubject;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.IEValue;
import de.desy.mst.libs.accelerator.Utility;
import de.desy.mst.libs.accelerator.view.CColor;
import de.desy.mst.libs.accelerator.view.CFrame;
import de.desy.mst.libs.accelerator.view.ComponentSubject;
import de.desy.mst.libs.accelerator.view.IViewComponent;
import de.desy.mst.libs.accelerator.view.IViewContainer;
import de.desy.mst.libs.accelerator.view.generic.RFButton;
import de.desy.mst.libs.accelerator.view.generic.RFLabel;
import de.desy.mst.libs.accelerator.view.generic.RFPanel;

/**
 * This displays a status label, two buttons and a before leading label.
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFSwitchArray.java 19706 2008-12-14 12:41:25Z walla $
 */
public abstract class RFSwitchArray implements IViewContainer
{
	/** The serial version UID, which is used by the java.io.Serializable
	 * interface. */
	public static final long serialVersionUID = 1070307234965134646l;

	/** Is the panel which holds all entries of the status in it. */
	private JPanel _panel;

	/** Is the icon of the possible drawn line between the columns. */
	private Icon _image;

	/** Is the map of the pairs IEData and IViewComponent for the
	 * button on. */
	protected Map<IEData, IViewComponent> _mapButtonOn;

	/** Is the map of the pairs IEData and IViewComponent for the
	 * button off. */
	protected Map<IEData, IViewComponent> _mapButtonOff;

	/** Is the map of the pairs IEData and IViewComponent for the
	 * label. */
	protected Map<IEData, IViewComponent> _mapLabel;

	/** Is the map of the pairs IEData and IViewComponent for the
	 * switch label. */
	protected Map<IEData, IViewComponent> _mapStatus;

	/**
	 * Constructs and initialize the two buttons and the label.
	 *
	 * @param dim_ Is the dimension if this container. It can also be
	 * 		{@code null}.
	 * @param rows_ This is the number of rows for each column.
	 * @param insets_ Is the gap between the different entries. The class of
	 * 		{@code Insets} allows to set a gap in all four directions. It
	 * 		can also be {@code null}.
	 * @param hasLine_ Is a boolean parameter which allows to draw a line
	 * 		between a group of elements. This divides different groups in
	 * 		columns by a line.
	 * @param strTitle_ This is the title of the switch container. This can
	 * 		also be {@code null}.
	 * @param element_ Is an array of the enumeration interfaces.
	 * @param action_ Is an array of the action commands.
	 */
	public RFSwitchArray(final Dimension dim_, final int rows_,
			final Insets insets_, final boolean hasLine_,
			final String strTitle_, final IEData[][] element_,
			final String[][] action_)
	{
		checkArguments(rows_, hasLine_, element_, action_);

		_mapButtonOn = new HashMap<IEData, IViewComponent>();
		_mapButtonOff = new HashMap<IEData, IViewComponent>();
		_mapLabel = new HashMap<IEData, IViewComponent>();
		_mapStatus = new HashMap<IEData, IViewComponent>();

		_image = new ImageIcon(getClass().getResource("/resources/icons/" +
		"SEPERATORLINE_VERTICAL.PNG"));

		if (dim_ != null)
			_panel = new RFPanel(new Dimension(dim_.width, dim_.height));
		else
			_panel = new JPanel();

		_panel.setLayout(new GridBagLayout());
		_panel.setOpaque(false);

		if (strTitle_ != null && strTitle_.length() > 0)
			_panel.setBorder(BorderFactory.createTitledBorder(strTitle_));
		else
			_panel.setBorder(BorderFactory.createLineBorder(CColor.GRAY_LIGHT));

		if (insets_ == null)
			initialize((rows_ < 1 || rows_ > 6)? 2 : rows_,
					new Insets(0,0,0,0), hasLine_, element_,
					action_);
		else
			initialize((rows_ < 1 || rows_ > 6)? 2 : rows_, insets_,
					hasLine_, element_, action_);
	}

	/**
	 * This will initialize the whole panel with all containers. All drawn
	 * container, which has the length of the delivered array of
	 * {@code data_}. Each column has a length of {@code rows_} lines or
	 * rows. If the length does not contain all possible positions the rest is
	 * filled with empty containers.
	 *
	 * @param rows_ This is the number of rows for each column.
	 * @param insets_ Is the gap between the different entries. The class of
	 * 		{@code Insets} allows to set a gap in all four directions.
	 * @param hasLine_ Is a boolean parameter which allows to draw a line
	 * 		between a group of elements. This divides different groups in
	 * 		columns by a line.
	 * @param data_ Is an array of the enumeration interface.
	 */
	private void initialize(final int rows_,
			final Insets insets_, final boolean hasLine_,
			final IEData[][] data_,
			final String[][] action_)
	{
		int counter = 0;
		int cols = ((data_.length / rows_))
			+ ((data_.length % rows_ == 0) ? 0 : 1);


		GridBagConstraints position = new GridBagConstraints();

		// each column is drawn here
		for(int i = 0, column = 0; i < cols; i++)
		{
			JPanel panel = new JPanel();
			// the columns is here set to 4, because I'm inserting three elements:
			// label, switch label and two buttons! The last two values are the
			// horizontal and vertical gap.
			panel.setLayout(new GridLayout(rows_, 4, 2, 2));

			// each row is drawn here
			for(int row = 0; row < rows_; row++)
			{
				if (counter >= data_.length)
					createEmptySwitch(panel);
				else
					createSwitch(panel, action_[counter], data_[counter]);
				counter+=1;
			}

			_panel.add(panel, createSwitchPosition(column, position));
			column++;

			if (counter < data_.length && hasLine_)
			{
		        _panel.add(createSeparator(column, position)
		        		, position);
				column++;
			}
		}

	}

	/**
	 * Creates a switch with two buttons: one for switch the device on and the
	 * other for off.
	 *
	 * @param panel_ The current panel in which the different components are
	 * 		added.
	 * @param action_ Is the action string for the button as an action command.
	 * @param data_ Is the data component element.
	 */
	private void createSwitch(final JPanel panel_, final String[] action_,
			final IEData[] data_)
	{
		IComponentSubject subject = ComponentSubject.getInstance();

		ButtonGroup group = new ButtonGroup();

		RFLabel label = new RFLabel(data_[0].getName(EName.NAME)
				+ ':', CFrame.FONT_SWITCH, null);
		label.setOpaque(false);
		_mapLabel.put(data_[0], label);

		RFLabel status = new RFLabel("***", CFrame.FONT_SWITCH, null);
		status.setBorder(BorderFactory.createLineBorder(CColor.BLACK, 2));
		status.setOpaque(false);
		status.setHorizontalAlignment(SwingConstants.CENTER);
		status.setVerticalAlignment(SwingConstants.CENTER);
		subject.register(data_[0], status);
		_mapStatus.put(data_[0], status);

		RFButton toggleON =
			new RFButton(data_[1].getName(EName.BUTTON)
					, CFrame.FONT_SWITCH, null);
		toggleON.setActionCommand(action_[0]);
		toggleON.setToolTipText("<html>" + Utility.getAddress(data_[1])
				+ "<br>Schaltet " + data_[1].getName(EName.NAME) + " auf "
				+ data_[1].getName(EName.BUTTON) + "</html>");
		group.add(toggleON);
		subject.register(data_[1], toggleON);
		_mapButtonOn.put(data_[1], toggleON);

		RFButton toggleOFF =
			new RFButton(data_[2].getName(EName.BUTTON)
					, CFrame.FONT_SWITCH, null);
		toggleOFF.setActionCommand(action_[1]);
		toggleOFF.setToolTipText("<html>" + Utility.getAddress(data_[2])
				+ "<br>Schaltet " + data_[2].getName(EName.NAME) + " auf "
				+ data_[2].getName(EName.BUTTON) + "</html>");
		group.add(toggleOFF);
		subject.register(data_[2], toggleOFF);
		_mapButtonOff.put(data_[2], toggleOFF);

		panel_.add(label);
		panel_.add(status);
		panel_.add(toggleON);
		panel_.add(toggleOFF);
	}

	/**
	 * Creates an empty switch.
	 *
	 * @param panel_ The panel which includes this switch.
	 */
	private void createEmptySwitch(final JPanel panel_)
	{
		panel_.add(new RFLabel("", CFrame.FONT_SWITCH, null));
		panel_.add(new RFLabel("", CFrame.FONT_SWITCH, null));
		panel_.add(new RFLabel("", CFrame.FONT_SWITCH, null));
		panel_.add(new RFLabel("", CFrame.FONT_SWITCH, null));
	}

	/**
	 * Create a separator.
	 *
	 * @param column_ Is column in which the separator should be drawn.
	 * @param position_ Is the grid bag constraint layout.
	 * @return Returns the label.
	 */
	private RFLabel createSeparator(final int column_,
			final GridBagConstraints position_)
	{
		// can also be solved via a JSeparator!!!!
		position_.gridx = column_;
		position_.gridy = 0;
		position_.anchor = GridBagConstraints.CENTER;
		position_.gridwidth = 1;
		position_.gridheight = 1;
		position_.insets = new Insets(2,2,2,2);
		position_.ipadx = 0;
		position_.ipady = 0;
		position_.weightx = 1.0;
		position_.weighty = 1.0;
		position_.fill = GridBagConstraints.NONE;
        RFLabel separator = new RFLabel(_image, null);
        separator.setOpaque(false);
        //JSeparator separator = new JSeparator(SwingConstants.HORIZONTAL);

        return separator;
	}

	/**
	 * Creates a grid bag constraints layout.
	 *
	 * @param column_ Is the column in which to draw with the grid bag
	 * 		constraints layout.
	 * @param position_ Is the delivered grid bag constraints layout.
	 * @return Is the returned grid bag constraints layout.
	 */
	private GridBagConstraints createSwitchPosition(final int column_,
			final GridBagConstraints position_)
	{
        position_.gridx = column_;
        position_.gridy = 0;
        position_.anchor = GridBagConstraints.LINE_START;
        position_.gridwidth = 1;
        position_.gridheight = 1;
        position_.insets = new Insets(2,2,2,2);
        position_.ipadx = 0;
        position_.ipady = 0;
        position_.weightx = 1.0;
        position_.weighty = 1.0;
        position_.fill = GridBagConstraints.NONE;

        return position_;
	}
	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#reset(de.desy.mst.libs.accelerator.IEValue)
	 */
	public void reset(final IEValue container_)
	{
		throw new RuntimeException("This method is currently not implemented.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getPanel()
	 */
	public JComponent getPanel()
	{
		return _panel;
	}


	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getBoolean(de.desy.mst.libs.accelerator.IEValue, de.desy.mst.libs.accelerator.EViewElement)
	 */
	public boolean getBoolean(final IEValue container_, final EViewElement element_)
	{
		throw new RuntimeException("This method is not implemented!");
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#getValue(de.desy.mst.libs.accelerator.IEValue)
	 */
	public Number getValue(final IEValue container_)
	{
		throw new RuntimeException("This method is not implemented!");
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#setBoolean(de.desy.mst.libs.accelerator.IEValue, de.desy.mst.libs.accelerator.EViewElement, boolean)
	 */
	public void setBoolean(final IEValue container_, final EViewElement element_,
			final boolean value_)
	{
		throw new RuntimeException("This method is not implemented!");
	}

	/**
	 * <p><u><i>ATTENTION:</u></i> THIS METHOD IS CURRENTLY NOT USED AND WILL
	 * THROW A RUNTIME EXCEPTION!<p>
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewContainer#setValue(de.desy.mst.libs.accelerator.IEValue, java.lang.Number)
	 */
	public void setValue(final IEValue container_, final Number value_)
	{
		throw new RuntimeException("This method is not implemented!");
	}

	/**
	 * This will check all arguments for there validity. If their exists one
	 * value which is invalid an {@code IllegalArgumentException} is thrown.
	 *
	 * @param rows_ This is the number of rows for each column.
	 * @param hasLine_ Is a boolean parameter which allows to draw a line
	 * 		between a group of elements. This divides different groups in
	 * 		columns by a line.
	 * @param element_ Is an array of the enumeration interfaces.
	 * @param action_ Is an array of the action commands.
	 */
	private void checkArguments(final int rows_, final boolean hasLine_,
			final IEData[][] element_,	final String[][] action_)
	{
		if (element_ == null)
			throw new NullPointerException("The data enumeration is null.");
		if (element_.length == 0)
			throw new IllegalArgumentException("The data enumeration shell" +
					" not be empty.");
		if (element_[0].length != 3)
			throw new IllegalArgumentException("An element of the data" +
					" enumeration shell have three elements.");

		if (action_ == null)
			throw new NullPointerException("The action enumeration is null.");
		if (action_.length == 0)
			throw new IllegalArgumentException("The action enumeration" +
					" shell not be empty.");
		if (action_[0].length != 2)
			throw new IllegalArgumentException("An element of the action" +
					" enumeration shell have two elements.");

		if (action_.length != element_.length)
			throw new IllegalArgumentException("All arrays (data enumerations," +
					" and the action commands) shell have the same length.");
	}

}
/* end of file */