/*
 *
 * File: RFButton.java
 * Date: Jan 25, 2007; 9:33:03 AM
 */
package de.desy.mst.libs.accelerator.view.generic;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionListener;
import java.util.EventListener;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.border.Border;

import de.desy.mst.libs.accelerator.EGraphicComponent;
import de.desy.mst.libs.accelerator.ESeverity;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.exception.OperationNotAllowedException;
import de.desy.mst.libs.accelerator.view.CFrame;
import de.desy.mst.libs.accelerator.view.IViewComponent;

/**
 * This is the manipulated {@code JButton} which allows beside calling
 * the constructor with its normal java values also to provide its dimension.
 * Moreover there are also there inherited methods from the class
 * {@code JComponent} which all will return the delivered dimension value, so
 * that there no resize will happen. There can also be delivered a new font.
 * If this value is not set the normal font is dialed.
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFButton.java 36465 2010-06-04 08:04:07Z walla $
 * @see javax.swing.JButton
 */
public final class RFButton extends JButton implements IViewComponent
{
	// --- CLASS FIELDS --------------------------------------------------------
	
	/** This is the current unique serial version UID. It helps the application
	 * to control serialization compatibility between versions. */
	final static long serialVersionUID = 599929006438538492L;

	/** Class name. */
	public final static String CLAZZ = RFButton.class.getSimpleName();
	
	// --- INSTANCE FIELDS -----------------------------------------------------
	
    /** This is the dimension of this container. */
    private Dimension _dim;

    /** The severity of the button component. This has no direct effect on the
     * GUI element. */
    private ESeverity _severity = ESeverity.NORMAL;

    // --- CONSTRUCTOR ---------------------------------------------------------
    
    /**
     * Constructs a button with an empty text field.
     *
	 * The dimension is currently {@code null} and therefore this instance can
	 * vary in its size. Also the default font is used.
	 * 
	 * @see RFButton#RFButton(Font, Dimension)
     */
    public RFButton()
    {
    	this(null);
    }
    
    /**
     * Constructs a button with the entire button text field.
     *
	 * The dimension is currently {@code null} and therefore this instance can
	 * vary in its size. Also the default font is used.
	 * 
	 * @param text_ Text string.
	 * @see RFButton#RFButton(String, Font, Dimension)
     */
	public RFButton(final String text_)
	{
		this(text_, null, null);
	}
    
    /**
     * Constructs a button with the given dimension.
     *
     * @param font_ Is the current font. This can also be null.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
     */
	public RFButton(final Font font_, final Dimension dim_)
	{
		initialization(dim_, font_);
    	debug();
	}

	/**
     * Constructs a button with the given dimension.
	 *
	 * @param icon_ The Icon image to display on the button.
     * @param font_ Is the current font. This can also be null.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	public RFButton(final Icon icon_, final Font font_, final Dimension dim_)
	{
		super(icon_);
		initialization(dim_, font_);
    	debug();
	}

	/**
     * Constructs a button with the given dimension.
	 *
	 * @param text_ The text of the button.
     * @param font_ Is the current font. This can also be null.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	public RFButton(final String text_, final Font font_, final Dimension dim_)
	{
		super(text_);
		initialization(dim_, font_);
    	debug();
	}

	/**
     * Constructs a button with the given dimension.
	 *
	 * @param action_ The {@code Action} used to specify the new button.
     * @param font_ Is the current font. This can also be null.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	public RFButton(final Action action_, final Font font_,
			final Dimension dim_)
	{
		super(action_);
		initialization(dim_, font_);
    	debug();
	}

	/**
     * Constructs a button with the given dimension.
	 *
	 * @param text_ The text of the button.
	 * @param icon_ The Icon image to display on the button.
     * @param font_ Is the current font. This can also be null.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	public RFButton(final String text_, final Icon icon_, final Font font_,
			final Dimension dim_)
	{
		super(text_, icon_);
		initialization(dim_, font_);
    	debug();
	}

	// --- INSTANCE METHODS ----------------------------------------------------

	/**
	 * Initialization.
	 *
     * @param font_ Is the current font. This can also be null.
     * @param dim_ This is the dimension of this container. If the dimension is 
     * 		{@code null} the size of the instance is taken via its super method
     * 		and so fore this instance isn't fixed in its size.
	 */
	private void initialization(
			final Dimension dim_, 
			final Font font_)
	{
		if (font_ != null)
			setFont(font_);
   		_dim = dim_;
	}

	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	
	@Override
	public Dimension getMinimumSize()
	{
		if (_dim == null)
			return super.getMinimumSize();
		return _dim;
	}

	@Override 
	public Dimension getMaximumSize()
	{
		if (_dim == null)
			return super.getMaximumSize();
		return _dim;
	}

	@Override 
	public Dimension getPreferredSize()
	{
		if (_dim == null)
			return super.getPreferredSize();
		return _dim;
	}
	
	@Override 
	public void setSize(final Dimension dim_)
	{
		_dim = dim_;
		super.setSize(dim_);
	}
	
	@Override
	public String toString()
	{
		return CLAZZ + "[text=" + this.getText() + ']';
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	
	@Override
	public void debug()
	{
		CFrame.DEBUG_BORDER(this);
	}

	@Override
	public void putSeverity(final ESeverity severity_)
	{
		_severity = severity_;
	}

	@Override
	public ESeverity fetchSeverity()
	{
		return _severity;
	}
	
	@Override
	public EGraphicComponent fetchGraphicComponent()
	{
		return EGraphicComponent.BUTTON_FIXED;
	}

	@Override
	public String fetchText(IEData view_)
	{
		return super.getText();
	}

	@Override
	public boolean isEnable()
	{
		return super.isEnabled();
	}

	@Override
	public boolean isVisual()
	{
		return super.isVisible();
	}

	@Override
	public boolean hasOpaque()
	{
		return super.isOpaque();
	}

	@Override
	public boolean isValueAdjusting(IEData view_)
	{
		return false;
	}

	@Override
	public void putListener(final EventListener listener_)
	{
		if (listener_ instanceof ActionListener)
			super.addActionListener((ActionListener) listener_);
		else
			throw new IllegalArgumentException("The listener model you are" +
					" adding doesn't exist for this instance: " +
					listener_.toString());
	}

	@Override
	public void putText(String label_, IEData view_)
	{
		super.setText(label_);
	}

	@Override
	public void putNumber(Number number_, IEData view_)
	{
		throw new OperationNotAllowedException("The operation is not allowed.");
	}

	@Override
	public void putEnable(boolean enable_)
	{
		super.setEnabled(enable_);
	}

	@Override
	public void putVisual(boolean visual_)
	{
		super.setVisible(visual_);
	}

	@Override
	public void putOpaque(boolean opaque_)
	{
		super.setOpaque(opaque_);
	}

	@Override
	public void putForeground(Color foreground_)
	{
		super.setForeground(foreground_);
	}

	@Override
	public void putBackground(Color background_)
	{
		super.setBackground(background_);
	}

	@Override
	public void putTooltip(String tooltip_)
	{
		this.setToolTipText(tooltip_);
	}

	@Override
	public void putSelected(boolean isSelected_)
	{
		throw new RuntimeException("This method makes no sense here!");
	}

	@Override
	public void putBorder(final Border border_)
	{
		super.setBorder(border_);
	}
}
/* end of file */