/*
 * File: CavityListener.java
 * Date: Jan 30, 2008; 2:51:13 PM
 */
package de.desy.mst.libs.accelerator.view.listener;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import de.desy.mst.libs.accelerator.CGeneralConstant;
import de.desy.mst.libs.accelerator.EViewElement;
import de.desy.mst.libs.accelerator.IComponentSubject;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.IEDataSendDataType;
import de.desy.mst.libs.accelerator.IEDataSendNumber;
import de.desy.mst.libs.accelerator.exception.TineException;
import de.desy.mst.libs.accelerator.model.DataExchange;
import de.desy.mst.libs.accelerator.model.DataExchange.ESend;
import de.desy.mst.libs.accelerator.view.ComponentSubject;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.tine.definitions.TErrorList;

/**
 * This is the listener class for switching a cavity button.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: CavityListener.java 19685 2008-12-12 15:40:45Z walla $
 */
public final class CavityListener implements ActionListener
{
	/** Is the maximum delay time. */
	public final static int MAX_DELAY_TIME = 60000;

	/** Is the minimum delay time. */
	public final static int MIN_DELAY_TIME = 1000;

	/** This is the data element which belongs to this listener class. */
	private IEData _data;

	/** Is the delay time during switching the button. */
	private int _delay;

	/** Is a list of data elements which are enabled during switching. */
	private List<IEData> _lstData;

	/**
	 * Initializes the cavity listener, which will send a value of 1 or 0 to
	 * the corresponding TINE path from the data enumeration {@code IEData}. The
	 * delay time is stored in the parameter {@code int}, which tells how long
	 * the function should wait after sending the value.
	 *
	 * @param data_ Is the sending enumeration data value. Should never be null.
	 * @param delay_ Is the wait time.
	 * @param lstData_ Is the list of data components which possibly will be
	 * 		enabled during switching it. This can also be {@code null}
	 */
	public CavityListener(final IEData data_, final int delay_,
			final List<IEData> lstData_)
	{
		validate(data_, delay_);
		initialize(data_, delay_, lstData_);
	}

	/**
	 * Validates the cavity listener class.
	 *
	 * @param data_ Is the sending enumeration data value. Should never be null.
	 * @param delay_ Is the wait time.
	 */
	private void validate(final IEData data_, final int delay_)
	{
		if (data_ == null)
			throw new NullPointerException("The IEData element is null.");
		if (!(data_ instanceof IEDataSendNumber ||
				data_ instanceof IEDataSendDataType))
			throw new IllegalArgumentException("Data is not an instance of "
					+ IEDataSendNumber.class.getSimpleName() + " or "
					+ IEDataSendDataType.class.getSimpleName());
		if (delay_ < 0 || delay_ > CGeneralConstant.TINE_WAIT_CAVITY_SWITCH)
			throw new IllegalArgumentException("The argument of delay is wrong: " + delay_);
	}

	/**
	 * Initialize the cavity listener class.
	 *
	 * @param data_ Is the sending enumeration data value. Should never be null.
	 * @param delay_ Is the wait time.
	 * @param lstData_ Is the list of data components which possibly will be
	 * 		enabled during switching it. This can also be {@code null}
	 */
	private void initialize(final IEData data_, final int delay_,
			final List<IEData> lstData_)
	{
		_data = data_;
		_delay = (delay_ < MIN_DELAY_TIME) ? MIN_DELAY_TIME :
			((delay_ > MAX_DELAY_TIME) ? MAX_DELAY_TIME : delay_);
		_lstData = lstData_;
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(final ActionEvent event_)
	{
		IComponentSubject subject = ComponentSubject.getInstance();

		if (_lstData != null)
			for(IEData data: _lstData)
				subject.notify(data, EViewElement.ENABLE, false);

		new Thread(new Runnable() {

			public void run()
			{
				Logger.SWApp.debug("sending cavity " + _data.toString());

				try
				{
					if (_data instanceof IEDataSendDataType)
						DataExchange.fireEventSendWithDataType(_data, _delay,
								ESend.CDI_SEND);
					else
						DataExchange.fireEventSendWithNumber(_data, _delay,
								ESend.CDI_SEND);
				}
				catch(TineException exception)
				{
					Logger.SWApp.error("Cavity listener: because of a" +
							" TINE failure the switching failed.", exception);
					JOptionPane.showMessageDialog(null,
							"Das Ausfhren der Cavity Schaltoperation schlug leider fehl!\n\n"
							+ "Es wurde die folgende Fehlermeldung durch TINE bertragen:\n"
							+ TErrorList.getErrorString(exception.getErrorCode()) + " (" + exception.getErrorCode() + ")\n\n",
							"Schreibfehler",
							JOptionPane.ERROR_MESSAGE);
				}
				finally
				{
					SwingUtilities.invokeLater(new Runnable() {
						IComponentSubject subject = ComponentSubject.getInstance();
						public void run() {
							if (_lstData != null)
							{
								for(IEData data: _lstData)
								{
									subject.notify(data, EViewElement.ENABLE, true);
								}
							}
						}
					});
				}
			}
		}, "CavitySet").start();
	}
}
/* end of file */