/**
 * Nov  5, 2007 - Nov 14, 2007	ws:	creation from scratch
 * Jan 10, 2008 - Jan 10, 2008	ws:	added setMaxBufferTime(...), EDT Flushing
 */
package de.desy.mst.libs.framework.mstapp.logging.appender;

import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;

import javax.swing.SwingUtilities;

/**
 * Wann soll der Buffer von ALogAppenderBuffered geflushed werden?
 * @author ws
 * @version 1.0.0
 */
public class FlushManager {
	private final IFlushable theFlushable;
	private final AtomicInteger  maxBufferSize;
	private final AtomicLong maxBufferTimeInMS;
	private final AtomicLong lastFlushDateInMS;
	private final AtomicBoolean closed;
	private final Runnable runnable;
//=============================================================================	
	/**
	 * @param theFlushable		not null
	 * @param maxBufferSize		sobald maxBufferSize erreicht ist: doFlush == true
	 * @param maxBufferTimeInMS nach maxBufferTimeInMS ms: ist doFlush == true
	 * 							Werte kleiner gleich null == kein flush Thread
	 */
	public FlushManager(final IFlushable theFlushable, final int maxBufferSize, final long maxBufferTimeInMS) {
		super();
		//System.out.println("FlushManager.XTor buff: " + maxBufferTimeInMS);
		if(theFlushable == null){ throw new NullPointerException("The flushable may not be null");}
		this.theFlushable		= theFlushable;
		this.maxBufferSize		= new AtomicInteger(maxBufferSize);
		this.maxBufferTimeInMS	= new AtomicLong(maxBufferTimeInMS);
		this.lastFlushDateInMS 	= new AtomicLong(0); 
		this.closed            	= new AtomicBoolean(false);
		this.runnable 			= createRunnable();
		createWatchThread();
		doFlush();
	}
//=============================================================================
	/**
	 * @param bufferSize
	 * @return true falls flush gerufen worden, sonst false
	 */
	public synchronized boolean fireNewBuffersize(final int bufferSize){
		if(	shouldFlushDueToBufferSize(bufferSize) 
			||
			shouldFlushDueToBufferTime()
			){
			doFlush();
			return true;
		}else{
			return false;
		}
	}
	public final IFlushable getTheFlushable() 	{return this.theFlushable;}
	
	public final int  getMaxBufferSize() 		{return this.maxBufferSize.get();}
	public final void setMaxBufferSize(final int size){this.maxBufferSize.set(size);}
	
	public final long getMaxBufferTimeInMS(){return this.maxBufferTimeInMS.get();}
	public final void setMaxBufferTime(final int msTime){
		synchronized(this){
			final boolean noActiveRunnable = noActiveRunnable();
			this.maxBufferTimeInMS.set(msTime);
			if(noActiveRunnable)
				createWatchThread();
		}
	}

	/**Falls nicht mehr gebraucht aufrufen (beended Thread & maxBufferSize = 1*/
	public void 	 close () {
		this.closed.set(true); 
		this.maxBufferSize.set(1);	
	}
	public boolean isClosed() {return this.closed.get();}
//=============================================================================
	/**pp*/final long getLastFlushDateInMS(){return this.lastFlushDateInMS.get();}
//=============================================================================
	private boolean shouldFlushDueToBufferSize(final int bufferSize) {
		return bufferSize >= this.maxBufferSize.get();
	}
	private boolean shouldFlushDueToBufferTime() {
		final long timeOut = this.maxBufferTimeInMS.get();
		return timeOut > 0	//else timeout disabled 
			&& calcBufferTimeInMS() >= timeOut;
	}
	private long calcBufferTimeInMS() {
		return (System.currentTimeMillis() - this.lastFlushDateInMS.get());
	}
	private synchronized void doFlush(){
		this.theFlushable.flush();
		this.lastFlushDateInMS.set(System.currentTimeMillis());
	}
	private void  createWatchThread(){ 
		if(noActiveRunnable())
			return;
		if(isClosed())
			return;
		SwingUtilities.invokeLater(this.runnable);
	}
	private boolean noActiveRunnable() {
		return getMaxBufferTimeInMS() < 0;
	}
	private Runnable createRunnable() {
		return new Runnable(){
			@Override public void run() {
				final long bufferTimeInMS = calcBufferTimeInMS();
				final long maxBufferTime = FlushManager.this.maxBufferTimeInMS.get();
				//System.out.println("FlushManager..run buff: " 
				//					+ bufferTimeInMS + ", " + maxBufferTime);
				if(bufferTimeInMS >= maxBufferTime){
					//System.out.println("FlushManager..run doFlush");
					doFlush();
				}
				createWatchThread();
			}
		};
	}
}
