/**
 *  Created on	Jul 19, 2006 - Aug 20, 2006
 *  Jan 11, 2008 - Jan 11, 2008	ws:	extracted: ILogProvider
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 * 	Jun 10, 2008 - Jun 10, 2008	ws: support reloadData()
 * 	Aug 12, 2008 - Aug 14, 2008	ws: gard getMessages() against spurious problems
 * 									default capacity from 100 to 500
 * 	Sep 23, 2008 - Sep 23, 2008 ws: ClientMgr fully synchronized
 * 	Oct 15, 2008 - Nov 24, 2008	ws:	getName(), getCapacityMinimum(), ...Max...
 * 	Nov  5, 2009 - Nov 11, 2009	ws:	try to fix deadlock threading issues
 *  Jan 11, 2010 - Jan 11, 2010	ws:	added clearBuffer() for testing purpuses
 */
package de.desy.mst.libs.framework.mstapp.logging.appender;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.desy.mst.libs.framework.mstapp.logging.ILogProvider;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.filter.FilterModus;
import de.desy.mst.libs.framework.mstapp.logging.filter.HauptLogFilter;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefManager;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;
import de.desy.mst.libs.framework.mstapp.util.xml.IXmlBeschreibung;

/**
 * Buffered Logging Eintraege ohne sie Tatsaechlich (selbst) rauszuschreiben. <br>
 * ILogAppenderKern koennen sich registrieren lassen.<br>
 * Folgende Events/Methoden werden delegiert:
 * <li> void doLog(final LogMsg msg)
 * <li> doLog(final List<LogMsg> msgList)
 * <li> void flush()
 * <br> d.h alle Events aus ILogAppenderKern
 * <br>
 * @author ws
 * @version 1.0.0
 */
public class LogAppenderBuffer implements ILogAppenderKern
										, IXmlBeschreibung
										, ILogProvider {
	public static final String NAME = "LogAppenderBuffer";
	public static final int    DEFAULT_CAPACITY = 500;

	private static final int CAPACITY_MIN = 5;	//a little Buffer for the file Output
	private static final int CAPACITY_RISKY_LARGE = 10000;	//should be just ok
	private static final int CAPACITY_MAX = Integer.MAX_VALUE/2;//dangerously large

	private class LogAppender extends ALogAppender{
		public LogAppender() {turnFilterOff();}	//default nicht filtern
		/**doLog nachdem es durch den filter positiv durch ist*/
		@Override public void doWrite(final LogMsg msg)	{
			LogAppenderBuffer.this.doWrite(msg);
		}
		@Override public void flush() 			{/*nothing to flush, so ignore*/}
		@Override public Element getBeschreibungAsElement(final Document document) {
			return getBeschreibungAsElement(document, NAME);
		}
		@Override public String getName() {return "In Memory Log Puffer";}
		//---------------------------------------------------------------------
		private void turnFilterOff() {this.getFilter().setModus(FilterModus.ALWAYS_PASS);}
		private synchronized void doFilter(final HauptLogFilter wegWirfFilter) {
			synchronized (LogAppenderBuffer.this.msgList) {
				final HauptLogFilter hauptLogFilter = (HauptLogFilter)this.getFilter();
				hauptLogFilter.set(wegWirfFilter);
				final List<LogMsg> msgListOri = getMsgList();
				final List<LogMsg> msgListCopy = new ArrayList<LogMsg> (msgListOri);
				msgListOri.clear();
				//setLastMessage(null);	//should always be a valid message after initializing
				LogAppenderBuffer.this.doLog(msgListCopy);
			}
		}
	}

	/**
	 * fully synchronized to avoid ConcurrentModificationException on clients
	 * alternative might be to put clients in atomic ref and (de)register on
	 * copies of the clients
	 */
	private class ClientMgr implements ILogAppenderKern{
		private final CopyOnWriteArrayList<ILogAppenderKern> clients = new CopyOnWriteArrayList<ILogAppenderKern>();
		//---------------------------------------------------------------------
		@Override public void doLog(final LogMsg msg) {
			for (final ILogAppenderKern client : this.clients) 
				doLog(client, msg);
		}
		@Override public void doLog(final List<LogMsg> msgList) {
			for (final ILogAppenderKern client : this.clients)
				doLog(client, msgList);
		}
		@Override public void flush() {
			for (final ILogAppenderKern client : this.clients)
				flush(client);
		}
		//---------------------------------------------------------------------
		//no synchronized for *register(...) routines
		//handled in CopyOnWriteArrayList (otherwise danger of deadlock)
		public boolean register(final ILogAppenderKern logAppenderKern){
			return logAppenderKern == null ? false :this.clients.addIfAbsent(logAppenderKern);
		}
		public boolean deregister  (final ILogAppenderKern logAppenderKern)	{return this.clients.remove(logAppenderKern);}
		public boolean isRegistered(final ILogAppenderKern appender)		{return this.clients.contains(appender);}
		//---------------------------------------------------------------------
		private void doLog(final ILogAppenderKern client, final LogMsg msg) {
			try {client.doLog(msg);} catch (final Exception e) {Logger.SWFW.warn(e);}
		}
		private void doLog(final ILogAppenderKern client, final List<LogMsg> msgList) {
			try {client.doLog(msgList);} catch (final Exception e) {Logger.SWFW.warn(e);}
		}
		private void flush(final ILogAppenderKern client) {
			try {client.flush();} catch (final Exception e) {Logger.SWFW.warn(e);}
		}
	}

//=============================================================================
	private final LogAppender logAppender;
	private final List<LogMsg> msgList;	//mods synched on this.msgList
	private volatile int    capacity; 
	private final ClientMgr clientMgr;
	private volatile LogMsg lastMessage;
//-----------------------------------------------------------------------------
	public LogAppenderBuffer() 								{this(DEFAULT_CAPACITY);}
	public LogAppenderBuffer(final int capacity) 			{this(capacity, null);}
	public LogAppenderBuffer(final List<LogMsg> messages)	{this(DEFAULT_CAPACITY, messages);}
	public LogAppenderBuffer(final int capacity, final List<LogMsg> messages) {
		this.logAppender = new LogAppender();
		this.msgList = (messages == null) ? new LinkedList<LogMsg>() : new LinkedList<LogMsg>(messages);
		this.capacity = capacity;
		this.clientMgr = new ClientMgr();
	}
//-----------------------------------------------------------------------------
	// Implementation von ILogAppender via Delegation zu internem getLogAppender()
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppenderKern#doLog(de.desy.mst.libs.framework.mstapp.logging.LogMsg)
	 */
	@Override public void doLog(final LogMsg msg)	{
		getLogAppender().doLog(msg);	//wird potentiell gefiltert und ruft dann doWrite(msg)
		getClientMgr().doLog(msg);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppenderKern#doLog(java.util.List)
	 */
	@Override public void doLog(final List<LogMsg> msgList){
		getLogAppender().doLog(msgList);
		getClientMgr().doLog(msgList);
	}
//-----------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.IFlushable#flush()
	 */
	@Override public void flush() {
		getLogAppender().flush();	//does nothing
		getClientMgr  ().flush();
	}
//-----------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.util.xml.IXmlBeschreibung#getBeschreibungAsElement(org.w3c.dom.Document)
	 */
	@Override public Element getBeschreibungAsElement(final Document document) {
		return getLogAppender().getBeschreibungAsElement(document);
	}
	/*
	 * (non-Javadoc)
	 *
	 * @see de.desy.mst.libs.framework.mstapp.util.xml.IXmlBeschreibung#getXMLBeschreibung(int)
	 */
	@Override public String getXMLBeschreibung(final int nTabs) {
		return getLogAppender().getXMLBeschreibung(nTabs);
	}
//-----------------------------------------------------------------------------
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogProvider#register(de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppenderKern)
	 */
	@Override public boolean register(final ILogAppenderKern logAppenderKern) {
		return getClientMgr().register(logAppenderKern);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogProvider#deregister(de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppenderKern)
	 */
	@Override public boolean deregister(final ILogAppenderKern logAppenderKern) {
		return getClientMgr().deregister(logAppenderKern);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogProvider#isRegistered(de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppenderKern)
	 */
	@Override public boolean isRegistered(final ILogAppenderKern appender) {
		return getClientMgr().isRegistered(appender);
	}


	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.ILogProvider#reloadData()
	 */
	@Override public void reloadAllData() {
		Logger.SWFW.debug("LogAppenderBuffer ignored wish to reload data.");
	}
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogProvider#getMessages()
	 */
	@Override public final List<LogMsg> getMessages() {
		try {
			synchronized (this.msgList) {
				return new ArrayList<LogMsg>(getMsgList());
			}
		} catch (final Exception e) {
			return getMessages();	//falls sich die Kopie wegen neuer Daten verschluckt
		}
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogProvider#getMessage(int)
	 */
	@Override public LogMsg getMessage(final int index) {
		synchronized (this.msgList) {
			return getMsgList().get(index);
		}
	}

	/**@return tatsaechliche Anzahl gespeicherter Logging Messages (nach dem Filtern)*/
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.ILogProvider#getSize()
	 */
	@Override public final int getSize() {
		synchronized (this.msgList) {
			return this.getMsgList().size();
		}
	}
//-----------------------------------------------------------------------------
	/**Puffer Groesse, immer > 0*/
	public final int getCapacity()	{return this.capacity;}
	/**Puffer Groesse, wird automatisch zwischen min und max gehalten*/
	public final void setCapacity(int capacity) {
		if(capacity < getCapacityMinimum())
			capacity = getCapacityMinimum();
		if( capacity > getCapacityMaximum())
			capacity = getCapacityMaximum();
		if( capacity == getCapacity())
			return;

		if(capacity < getCapacity())
			flush();	//sicherstellen dass nichts unnoetig verloren geht
		this.capacity = capacity;
		adjustListSize();
	}
	public final LogMsg getLastMessage(){return this.lastMessage;}

	/**nicht den Originalfilter, also haben Aenderungen nur einen effekt wenn man explizit doFilter(filter) aufruft*/
	public synchronized final HauptLogFilter getFilterClone()	{return new HauptLogFilter((HauptLogFilter)getLogAppender().getFilter());}
	/**aehnlich setFilter(filter), nur dass intern ein deep Clone verwended wird, der Buffer wird neu gefiltert und kuenftig erst mal immer so*/
	public synchronized final void        doFilter(final HauptLogFilter wegWirfFilter) {
		getLogAppender().doFilter(wegWirfFilter);
		getClientMgr().flush();	//alte Daten evtl nicht mehr entsprechend dem Filter
	}
	public void registerPreference(final IPrefManager prefMgr, final IPreferences parent) {
		prefMgr.register(new PrefLoggingAppenderBuffer(this, parent));
	}
	public int getCapacityDefault() 	{return DEFAULT_CAPACITY;}
	public int getCapacityMinimum() 	{return CAPACITY_MIN;}
	public int getCapacityMaximum() 	{return CAPACITY_MAX;}
	public int getCapacityRiskyLarge()	{return CAPACITY_RISKY_LARGE;}

	/**pp for test purpuses only*/ void clearBuffer() { 
		flush();
		this.msgList.clear();
	}
//-----------------------------------------------------------------------------
	private synchronized void doWrite(final LogMsg msg) {
		if(msg != null)
			setLastMessage(msg);
		
		synchronized (this.msgList) {
			getMsgList().add(msg);
			adjustListSize();
		}
	}
	private void adjustListSize() {
		synchronized (this.msgList) {
			final List<LogMsg> msgList = this.getMsgList();
			while (	   msgList.size() > 0
					&& msgList.size() > this.getCapacity()) {
				msgList.remove(0);	//von vorne entfernen
			}
		}
	}

	private final ClientMgr 	getClientMgr() 	{return this.clientMgr;}
	private final LogAppender 	getLogAppender(){return this.logAppender;}
	private final List<LogMsg>	getMsgList()	{
		synchronized (this.msgList) {
			return this.msgList;
		}
	}
	private final void setLastMessage(final LogMsg msg)	{
		this.lastMessage = msg;
	}
}
