
package de.desy.mst.libs.ser.launcher;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.concurrent.Callable;
import java.util.logging.Level;

import de.desy.mst.libs.ser.launcher.cache.CacheFactory;
import de.desy.mst.libs.ser.launcher.cache.IStartable;
import de.desy.mst.libs.ser.launcher.gui.MainFrm;

/**
 * @author labudda
 *
 */
public class Launcher implements Runnable, Callable<IStartable> {
	private final static java.util.logging.Logger log = java.util.logging.Logger.getLogger(Launcher.class.getName());

	private final static String CAMframeControlClassName = "de.desy.mst.libs.framework.mstapp.util.camInstrumentation.CAMframeControl";
	private final static boolean librariesAvailable = Launcher.checkClasspath();
	private enum EError {
		NoError,
		ArgumentError,
		RuntimeError
	}
	private final java.util.List<String> hrefs; 
	/*
	 * Optionen der Applikation. Alle optionen und argumente werden auf lcase konvertiert
	 * optionen koennen als Kommandozeilenargument oder als System property bzw. Umgebungsvariable mit dem Prefix LAUNCHER_ definiert werden.
	 * stayonsuccess - beendet die Java Applikation nicht, wenn die Applikation aus dem cache erfolgreich gestartet wurde
	 * stayonfailure - beendet die Java Applikation nicht, wenn die Applikation aus dem cache nicht gestartet werden konnte
	 * useAppCacheFolder - Speichert alle definierten Dateien im selben cache-verzeichnis wie die erste Datei  
	 */
	/**
	 * @param args defne href to download.
	 * arg[0] defines local cache folder name and startable
	 * Attention!! Blank will break one argument into two arguments
	 * So, blanks are prohibited
	 */
	public static void main(String[] args) {
		for (Object property : System.getProperties().keySet()) {
			Launcher.log.finest("detected system property " + property.toString() + " defined: " + System.getProperties().get(property).toString());
		}
		for (Object var : System.getenv().keySet()) {
				Launcher.log.finest("detected system environment " + var.toString() + " defined: " + System.getenv().get(var).toString());
		}
		// output known option
		for (EOptions option : EOptions.values()) {
			Launcher.log.finest("Supported option " + option.name() + " defined: " + option.isDefined());
		}
		javax.swing.SwingUtilities.invokeLater((new Launcher(args)));
	}
	
	/**
	 * @return
	 */
	private static boolean checkClasspath() {
		try {
			Class.forName("de.desy.tine.types.TCompoundDataObject");
			Class.forName(Launcher.CAMframeControlClassName);
			return true;
		} catch (ClassNotFoundException e) {
			return false;
		}
	}

	/**
	 * @param args of main
	 */
	public Launcher(String[] args) {
		this.hrefs = new java.util.LinkedList<String>();
		for (String arg : args) {
			hrefs.add(arg);
		}
	}

	/**
	 * @param hrefs to download. First element will be started
	 */
	public Launcher(java.util.Collection<String> hrefs) {
		this.hrefs = new java.util.LinkedList<String>(hrefs);
	}

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {

		EError detectedError = EError.NoError;
		Process process = null;
		try {
			if (this.hrefs.size() > 0) {
				final IStartable startable = this.call();
				process = startable.start();
				if (Launcher.librariesAvailable) {
					Launcher.doDaemonHandshake();
				} else {
					Launcher.log.info("de.desy.mst.libs.framework.mstapp.util.camInstrumentation.CAMframeControl while launching " + startable.getLocalPath());
				} 
				Launcher.log.info("Process was requested to launch. Any more output is for debug purpose.");
			} else {
				Launcher.log.warning("Invalid arguments: No launchable application defined");
				detectedError = EError.ArgumentError;
			}
		} catch (Exception e) {
			Launcher.log.severe(e.getMessage());
			detectedError = EError.RuntimeError;
		}
		final boolean stay = 
			(detectedError == EError.NoError && EOptions.StayOnSuccess.isDefined())
			|| (detectedError != EError.NoError && EOptions.StayOnFailure.isDefined());
		if (stay) {
			// more logging
			if (process == null) {
				Launcher.log.warning("A null process shall never be available on launcher stay option");
			} else {
				try {
					Integer exitCode  = null;
					{
						Launcher.logInputStream(process.getErrorStream(), java.util.logging.Level.SEVERE);
						Launcher.logInputStream(process.getInputStream(), java.util.logging.Level.INFO);
						exitCode = Launcher.getExitCode(process);
					} while (exitCode == null);
					Launcher.log.info("Exit code: " + exitCode);
				} catch (IOException e) {
					Launcher.log.warning(e.getMessage());
				}
			}
		} else {
			MainFrm.getInstance().waitAndClose(detectedError.ordinal());
		}
	}

	/**
	 * 
	 */
	private static void doDaemonHandshake() {
		try {
			Class<?> cam = Class.forName(Launcher.CAMframeControlClassName);
			
			final Class<?>[] parameter = {Boolean.TYPE};
			for (java.lang.reflect.Method m: cam.getDeclaredMethods()) {
				System.out.println(m.toString());
				Class<?>[] p = m.getParameterTypes();
				p.toString();
			}
			cam.getDeclaredMethod("doDaemonHandshake", parameter);
			java.lang.reflect.Method doDaemonHandshake = cam.getMethod("doDaemonHandshake", parameter);
			doDaemonHandshake.invoke(cam, true);
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	/**
	 * @param process to check
	 * @return process exit code on terminated process or null
	 */
	private static Integer getExitCode(Process process) {
		try {
			return process.exitValue();
		} catch (java.lang.IllegalThreadStateException e) {
			// any better way to detect terminated process?
			// anyhow, spend some time
			synchronized (Launcher.class) {
				try {
					Launcher.class.wait(1000);
				} catch (InterruptedException e1) {
					Launcher.log.warning(e1.getMessage());
					e1.printStackTrace();
				}
			}
			return null;
		}
		
	}

	/**
	 * @param stream to read from
	 * @param level how to log
	 * @throws IOException 
	 */
	private static void logInputStream(InputStream stream, Level level) throws IOException {
		final StringBuffer processOutput = new StringBuffer();
		if (stream == null) {
			// stream might be null on NullProcess by option UseOpen
		} else {
			while (stream.available() > 0) {
				processOutput.append((char) stream.read());
			}
		}
		if (processOutput.length() > 0) {
			Launcher.log.log(level, processOutput.toString());
		} else {
			// no output
		}
	}

	/* (non-Javadoc)
	 * @see java.util.concurrent.Callable#call()
	 */
	@Override
	public IStartable call() throws Exception {
		final int firstCacheIndex;
		final CacheFactory firstItem;
		final IStartable startable;
		if (EOptions.UseNativeCommand.isDefined()) {
			firstCacheIndex = 1;
			if (this.hrefs.size() > 1) {
				firstItem = new CacheFactory(this.hrefs.get(1));
			} else {
				// nothing to cache
				firstItem = null;
			}
			final String nativeApp = this.hrefs.get(0);
			MainFrm.getInstance().appendText("Starte der nativen Applikation '" + nativeApp + "' mit der Datei '" + firstItem + "' definiert.");
			startable = new NativeStartable(nativeApp, firstItem);
		} else {
			firstCacheIndex = 0;
			firstItem = new CacheFactory(this.hrefs.get(0));
			startable = firstItem.getStartable();
		}
		for (int pathIndex = firstCacheIndex; pathIndex <  this.hrefs.size(); pathIndex++) {
			final String path = this.hrefs.get(pathIndex);
			firstItem.create(path);
		}
		return startable;
	}
}