/**
 * 
 * Last changed: $Date$
 * By: $Author$
 * URL: $HeadURL$
 * $Revision$ 
 *
 */
package de.desy.mst.libs.ser.launcher.cache;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Date;
import java.util.prefs.Preferences;

import de.desy.mst.libs.ser.launcher.Launcher;


/**
 * @author labudda
 * Organisiert das caching einer Datei
 * Die Position des caches wird definiert durch 
 * 1. Sourcecode
 * 2. System preference
 * 3. user preference
 * Der vollstaendige Name der Preference lautet
 * de/desy/mst/libs/ser/launcher/cache location
 * Be aware of java syntax to escape '\'
 *
 */
class CacheItem implements ICacheItem {
	private final static java.util.logging.Logger log = java.util.logging.Logger.getLogger(CacheItem.class.getName());

	/**
	 * Vollstaendige Pfad zum Item im cache
	 */
	private final String cacheLocation;
	private final boolean isAvailable;
	CacheItem(final String url) throws MalformedURLException {
		this(new java.net.URL(url));
	}

	/**
	 * Definiert die Quelle der lokalen Datei via  URL
	 * Als Protokoll ist nur http unterstuetzt  
	 * @param url
	 */
	CacheItem(final java.net.URL url) throws MalformedURLException {
		this(url, CacheItem.getBaseCacheLocation() + CacheItem.urlToCache(url));
	}

	/**
	 * @param url of source file
	 * @param cachedItem
	 * @throws MalformedURLException
	 */
	CacheItem(final String url, final ICacheItem cachedItem) throws MalformedURLException {
		this(new java.net.URL(url), cachedItem.getCachedItemFolder() + "/" + CacheItem.getLastPathElement(url));
	}
	
	/**
	 * @param url
	 * @return the last item of a url after '/'
	 */
	private static String getLastPathElement(String url) {
		final String[] items = url.split("/");
		return items[items.length - 1];
	}

	/**
	 * @param url
	 * @param cacheLocation: file created on local disk
	 * @throws MalformedURLException
	 */
	private CacheItem(final java.net.URL url, final String cacheLocation) throws MalformedURLException{
		if (url == null) {
			throw new MalformedURLException("null is not a valid url");
		} else if (!"http".equalsIgnoreCase(url.getProtocol())) {
			throw new MalformedURLException(url.getProtocol() + " wird nicht unterstuetzt.");
		} else if (!cacheLocation.startsWith(CacheItem.getBaseCacheLocation())) {
			throw new java.lang.IllegalArgumentException("cache location must be inside base location");
		} else {
			this.cacheLocation = cacheLocation;
			final long start = System.currentTimeMillis();
			CacheItem.log.info(" Going to download url " + url.getFile());
			this.isAvailable = CacheItem.download(url, this.cacheLocation);
			CacheItem.log.info("Finished to download url " + url.getFile() + " in " + (System.currentTimeMillis() - start) + " ms");
		}
	}
	
	/**
	 * Aus einer url wird ein Resourcen Stream erzeugt
	 * @param url
	 * @param cacheLocation
	 */
	private static boolean download(URL url, String cacheLocation) {
		try {
			if (CacheItem.isResourceUpToDate(url, cacheLocation)) {
				CacheItem.log.finer(url.getFile() + " is up to date");
			} else {
				final long lastModified = url.openConnection().getLastModified();
				final ResourceStream resourceStream = new ResourceStream(url.openStream(), lastModified);
				CacheItem.download(resourceStream, cacheLocation);
			}
			return true;
		} catch (IOException e) {
			CacheItem.log.warning("Unable to download " + url.getPath() + ": " + e.getClass().getSimpleName());
			return false;
		}
		
	}

	/**
	 * @param url
	 * @param cacheLocation
	 * @return true, wenn die lokale Datei aktuell ist
	 */
	private static boolean isResourceUpToDate(URL url, String cacheLocation) {
		try {
			// final long srcTimestamp = url.openConnection().getDate();
			final long srcTimestamp = url.openConnection().getLastModified();
			final long destTimestamp = (new java.io.File(cacheLocation)).lastModified();
			if (srcTimestamp == 0L) {
				CacheItem.log.info("Unknown timestamp of " + url.getFile());
				return false;
			} else if (destTimestamp == 0) {
				// destination does not exist
				CacheItem.log.warning("Non existing destination of: " + url.getFile());
				return false;
			} else {
				final long delta = srcTimestamp - destTimestamp;
				CacheItem.log.fine(url.getFile() + " src timestamp: " + new Date(srcTimestamp) + "; dest timestamp: " + new Date(destTimestamp));
				CacheItem.log.fine(url.getFile() + " src timestamp: " + srcTimestamp + "; dest timestamp: " + destTimestamp + "; delta: " + delta);
				return delta <= 0;
			}
		} catch (IOException e) {
			CacheItem.log.warning(e.getMessage());
			return false;
		}
		
	}

	/**
	 * Laedt die Datei vom Server und speichert sie im Cache
	 * Fehler werden ignoriert. Entweder befindet sich die DAtei bereits im Cache 
	 * oder bei Startup der Datei wird ein Fehler generiert 
	 * @param resourceStream
	 * @param cacheLocation 
	 * @return true on success
	 */
	private static boolean download(IResource resourceStream, String cacheLocation) {
		try {
			resourceStream.copyResourcesTo(cacheLocation);
			return true;
		} catch (IOException e) {
			CacheItem.log.warning(e.getMessage());
			return false;
		}
	}

	/**
	 * Aus einer URL wird ein foldername erzeugt.
	 * Ueber diesen Foldernamen wird die Position des lokalen Elementes definiert.
	 * D.h. ueber die URL wird eine eindeutige Position definiert
	 * @param url
	 * @return Position im Cache
	 */
	private static String urlToCache(URL url) {
		final String path = url.getHost() + url.getPath();
		final String name = CacheItem.getLastPathElement(url.getPath());
		final String item = path + java.io.File.separator + name; 
		return item;
	}

	/**
	 * @param resourceClass containing item to cache
	 */
	CacheItem(Class<? extends Launcher> resourceClass) {
		final String itemName = resourceClass.getSimpleName() + ".exe";
		final String find = "/resources/" + itemName;
		CacheItem.log.finest("load resources of " + resourceClass.getName());
		java.io.InputStream stream = resourceClass.getResourceAsStream(find);
		if (stream == null) {
			CacheItem.log.fine(find + " not found");
			this.cacheLocation = null;
			this.isAvailable = false;
		} else {
			final IResource root;
			this.cacheLocation = CacheItem.getBaseCacheLocation() 
			+ CacheItem.packageToFolder(resourceClass.getName())
			+ itemName; 
			this.isAvailable = CacheItem.download(new ResourceStream(stream), this.cacheLocation);
			root = new ResourceStream(stream);
			try {
				root.copyResourcesTo(this.cacheLocation);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.ICacheItem#getCachedItemPath()
	 */
	public String getCachedItemPath(){
		return this.cacheLocation;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.ICacheItem#getCachedItemFolder()
	 */
	public String getCachedItemFolder(){
		final java.io.File file = new java.io.File(this.cacheLocation); 
		return file.getParent();
	}

	/**
	 * @param name of class to convert to path fragment
	 * @return path fragment
	 */
	private static String packageToFolder(String className) {
		final String[] names = className.split("\\.");
		final StringBuffer result = new StringBuffer(className.length());
		for (String name : names) {
			result.append(name);
			result.append(java.io.File.separatorChar);
					
		}
		return result.toString();
	}



	
	/**
	 * @return Folder of cache location including last seperator
	 */
	private static String getBaseCacheLocation(){
		final String sourcecodeDefault = System.getProperty("user.home") 
		+ java.io.File.separator
		+ "Local Settings"
		+ java.io.File.separator
		+ "Application Data"
		+ java.io.File.separator
		+ "desy-mca"
		+ java.io.File.separator;
		CacheItem.log.finest("cache location defined by sourcecode: " + sourcecodeDefault);
		final String systemCacheLocation = CacheItem.getLocationPreference(java.util.prefs.Preferences.systemRoot(), sourcecodeDefault);
		CacheItem.log.finest("cache location defined by system pref: " + systemCacheLocation);
		final String userCacheLocation = CacheItem.getLocationPreference(java.util.prefs.Preferences.userRoot(), systemCacheLocation);
		CacheItem.log.finest("cache location defined by user pref: " + userCacheLocation);
		final String cacheLocation;
		if (userCacheLocation.endsWith(java.io.File.separator)) {
			cacheLocation = userCacheLocation;
		} else {
			cacheLocation = userCacheLocation + java.io.File.separator;
		}
		CacheItem.log.finer("Use " + cacheLocation);
		return cacheLocation;
	}

	/**
	 * @param user or system preferences root 
	 * @param defaultValue
	 * @return preference or default value
	 */
	private static String getLocationPreference(Preferences preferences, String defaultValue) {
		return preferences.node("de").node("desy").node("mst").node("libs").node("ser").node("launcher").node("cache").get("location", defaultValue);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.ICacheItem#getStartable()
	 */
	public IStartable getStartable() {
		return new Startable(this);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.ICacheItem#isAvailable()
	 */
	@Override
	public boolean isAvailable() {
		return this.isAvailable;
	}

}
