
package de.desy.mst.libs.ser.launcher.gui;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;

/**
 * Die Klasse MainFrm erzeugt das Info-Fenster,
 * fuer die Programmausgaben.
 *
 * @author szczesny
 * 
 * Aenderungen:
 * 09. Sept. 2015: AL:	Singleton eingefuehrt
 * 						Klassenkonstanten statt Objektkonstanten
 * 						serialVersionUID
 * 						Logging integiert
 * 						Properties des JFrame aus dem Zugriff entfernt 
 * 						waitAndClose mit ExitCode Argument aus Gruenden der kompatibilitaet mit Vorversion 
 */
public class MainFrm extends JFrame implements IMainFrm {
    
    /**
	 * Nur um die Compilerwarnungen zu befriedigen
	 * Im Detail ist nicht vorgesehen diese Klase zu serialisieren  
	 */
	private static final long serialVersionUID = 1L;
	private final static String VERSION = "v0.2";
    private final static long WAITTIME = 5000; //ms
    
    /*
     * Logger wird aktuell konfiguriert ueber die Datei logging.properties 
     * Global befindet sich diese Datei im Verzeichnis "java.home"
     * individuell kann sie via VM Parameter -Djava.util.logging.config.file definiert werden
     * Bsp. -Djava.util.logging.config.file=./logging.properties
     * Statischer Logger i nder Klasse ist ggf. zu diskutieren.
     */
    private final static java.util.logging.Logger log = java.util.logging.Logger.getLogger(MainFrm.class.getName()); 
    
    private final JTextArea outPutText;
    private final JScrollPane sp;
    
    private volatile boolean close = true;
    
    /*
     * Klasse auf ein singleton umgestellt um nicht staendig ein globales Objekt 
     * der Klasse im Nutzercode mitfuehren zu muessen.
     * ggf. kann ueberlegt werden, ob die instance erst bei Bedarf 
     * durch getInstance() erzeugt werden solte
     */
    private final static IMainFrm instace = new MainFrm();
    public static IMainFrm getInstance() {
    	return MainFrm.instace;
    }
    
    /**
     * Konstruktor
     */
    private MainFrm() {
        super();
        
        this.setTitle("RunScript " + VERSION);
        this.setSize(400, 300);
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        outPutText = new JTextArea();
        outPutText.setLineWrap(true);
        outPutText.setWrapStyleWord(true);
        outPutText.setEditable(false);
        
        sp = new JScrollPane(outPutText);
        this.add(sp);
        
        this.setVisible(true);
        this.toFront();
    }
    
    /* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.gui.IMainFrm#appendText(java.lang.String)
	 */
    @Override
	public void appendText(final String Text) {
    	MainFrm.log.info(Text);
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                outPutText.append(Text + "\r\n");
            }
        });
    }
    /* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.gui.IMainFrm#appendError(java.lang.String)
	 */
    @Override
	public void appendError(final String ErrorText) {
    	MainFrm.log.severe(ErrorText);
        close = false;
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                outPutText.append("====> ERROR: " + ErrorText + "\r\n");
            }
        });
    }
    
    /* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.gui.IMainFrm#waitAndClose()
	 */
    @Override
	public void waitAndClose() {
    	this.waitAndClose(0);
    }

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.gui.IMainFrm#waitAndClose(int)
	 */
	@Override
	public void waitAndClose(final int ExitCode) {
        if(close) {
            appendText("\r\nLauncher info window will close in 5 sec.");
            Thread thWait = new Thread(new Runnable() {
                @Override
                public void run() {
                    try {
                        Thread.sleep(WAITTIME);
                        System.exit(ExitCode);
                    } catch (InterruptedException ex) {
                        appendError(ex.toString() + "[MainFrm.java -> waitAndClose()]");
                    }
                }
            });
            thWait.start();
        } else {
            appendText("\r\nAn error occurred!\r\nRunScript info window will not close automatically!");
        }
	}
    
}

